# Locate Delta3D
# This module defines
# DTCORE_LIBRARY
# DELTA3D_EXT_DIR
# DELTA3D_FOUND, if false, do not try to link to gdal 
# DELTA3D_INCLUDE_DIR, where to find the headers
#
# $DELTA3D_DIR is an environment variable that would
# correspond to the ./configure --prefix=$DELTA3D
#
# Created by David Guthrie. 

FIND_PATH(DELTA_DIR dtCore/dt.h
    PATHS
    ${CMAKE_SOURCE_DIR}/../delta3d
    $ENV{DELTA_ROOT}
    $ENV{DELTA_INC}
    ~/Library/Frameworks
    /Library/Frameworks
    /usr/local
    /usr
    /sw # Fink
    /opt/local # DarwinPorts
    /opt/csw # Blastwave
    /opt
    [HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControl\\Control\\Session\ Manager\\Environment;DELTA_ROOT]
    /usr/freeware/include
)

FIND_PATH(DELTA3D_INCLUDE_DIR NAMES dtCore/dt.h
    PATHS
       ${DELTA_DIR}
    PATH_SUFFIXES
       inc
       include
)

FIND_PATH(DELTA3D_LIB_DIR NAMES libdtCore.so libdtCore.dylib dtCore.dll .
    PATHS
       ${DELTA_DIR}
       /usr/local/lib
       /usr/local/lib64
    PATH_SUFFIXES
       Build/lib
       Debug/lib
       Release/lib
       lib
#    NO_DEFAULT_PATH
)

FIND_PATH( DELTA3D_EXT_DIR inc
    ${DELTA_DIR}/ext
)

SET(DTUTIL_LIBRARY dtUtil)
SET(DTCORE_LIBRARY dtCore)
SET(DTABC_LIBRARY dtABC)
SET(DTDAL_LIBRARY dtDAL)
#SET(DTDIRECTOR_LIBRARY dtDirector)
#SET(DTDIRECTORQT_LIBRARY dtDirectorQt)
SET(DTAI_LIBRARY dtAI)
SET(DTGAME_LIBRARY dtGame)
SET(DTAUDIO_LIBRARY dtAudio)
SET(DTANIM_LIBRARY dtAnim)
SET(DTGUI_LIBRARY dtGUI)
SET(DTINSPECTORQT_LIBRARY dtInspectorQt)
SET(DTSCRIPT_LIBRARY dtScript)
SET(DTTERRAIN_LIBRARY dtTerrain)
SET(DTNET_LIBRARY dtNet)
SET(DTNETGM_LIBRARY dtNetGM)
SET(DTHLAGM_LIBRARY dtHLAGM)
SET(DTDIS_LIBRARY dtDIS)
SET(DTINPUT_PLIB_LIBRARY dtInputPLIB)
SET(DTACTORS_LIBRARY dtActors)
SET(DTANIM_LIBRARY dtAnim)
SET(DTLMS_LIBRARY dtLMS)
SET(DTQT_LIBRARY dtQt)
SET(TEST_ACTOR_LIBRARY testActorLibrary)
SET(TEST_GAME_ACTOR_LIBRARY testGameActorLibrary)
SET(FIREFIGHTER_DEMO_LIBRARY fireFighter)

# Seems to be the fastest way to include the proper Delta3D libraries
# Including individual DTCORE_LIBRARY e.g. does not work. It resolves
# to dtCore instead of the complete file name.
SET(DELTA3D_LIBRARIES 
  ${DELTA3D_LIB_DIR}/libdtCore.so
  ${DELTA3D_LIB_DIR}/libdtUtil.so
  ${DELTA3D_LIB_DIR}/libdtABC.so
  ${DELTA3D_LIB_DIR}/libdtDAL.so
#  ${DELTA3D_LIB_DIR}/libdtDirector.so
  ${DELTA3D_LIB_DIR}/libdtGame.so
  ${DELTA3D_LIB_DIR}/libdtGUI.so
  ${DELTA3D_LIB_DIR}/libdtAnim.so
)

IF (WIN32)
    SET(DTUTIL_LIBRARY_DEBUG dtUtild)
    SET(DTCORE_LIBRARY_DEBUG dtCored)
    SET(DTABC_LIBRARY_DEBUG dtABCd)
    SET(DTDAL_LIBRARY_DEBUG dtDALd)
#    SET(DTDIRECTOR_LIBRARY_DEBUG dtDirectord)
#    SET(DTDIRECTORQT_LIBRARY_DEBUG dtDirectorQtd)
    SET(DTAI_LIBRARY_DEBUG dtAId)
    SET(DTGAME_LIBRARY_DEBUG dtGamed)
    SET(DTAUDIO_LIBRARY_DEBUG dtAudiod)
    SET(DTANIM_LIBRARY_DEBUG dtAnimd)
    SET(DTGUI_LIBRARY_DEBUG dtGUId)
    SET(DTINSPECTORQT_LIBRARY_DEBUG dtInspectorQtd)
    SET(DTSCRIPT_LIBRARY_DEBUG dtScriptd)
    SET(DTTERRAIN_LIBRARY_DEBUG dtTerraind)
    SET(DTNET_LIBRARY_DEBUG dtNetd)
    SET(DTNETGM_LIBRARY_DEBUG dtNetGMd)
    SET(DTHLAGM_LIBRARY_DEBUG dtHLAGMd)
    SET(DTDIS_LIBRARY_DEBUG dtDISd)
    SET(DTINPUT_PLIB_LIBRARY_DEBUG dtInputPLIBd)
    SET(DTACTORS_LIBRARY_DEBUG dtActorsd)
    SET(DTANIM_LIBRARY_DEBUG dtAnimd)
    SET(DTLMS_LIBRARY_DEBUG dtLMSd)
    SET(DTQT_LIBRARY_DEBUG dtQtd)
    SET(TEST_ACTOR_LIBRARY_DEBUG testActorLibraryd)
    SET(TEST_GAME_ACTOR_LIBRARY_DEBUG testGameActorLibraryd)
    SET(FIREFIGHTER_DEMO_LIBRARY_DEBUG fireFighterd)
ENDIF (WIN32)


SET(DELTA3D_FOUND "NO")
IF(DTCORE_LIBRARY AND DELTA3D_INCLUDE_DIR AND DELTA3D_LIB_DIR)
    SET(DELTA3D_FOUND "YES")
ENDIF(DTCORE_LIBRARY AND DELTA3D_INCLUDE_DIR AND DELTA3D_LIB_DIR)
