/*
 * multiActuator.h
 *
 * Actuator Container. All actuators in this container are controlled identically.
 * Actuator parameters like position or velocity are taken from the first actuator in the list. The actuators
 * in the list should be of the same type.
 *
 *  Created on: Dec 1, 2008
 *      Author: winkler
 */

#ifndef MULTIACTUATOR_H_
#define MULTIACTUATOR_H_

#include <srCore/actuator/actuatorBase.h>

namespace srCore {

class MultiActuator: public ActuatorBase {
public:
	MultiActuator(dtCore::RefPtr<ActuatorBase> firstElement, const std::string name);
	virtual ~MultiActuator();

	void init() {

	};

	//! add actuator to container
	void addActuator(dtCore::RefPtr<ActuatorBase> actuator);

	void registerActuator();

	//! Sets ode parameters for the actuators in the list. For more Information, see www.ode.org/ode-latest-user-guide.org.
	void setODEParameter(int parameter, float value);

	float getODEParameter(int parameter);

	void setParameter(const std::string &name, ParameterReturnValue  parameter);

	void getParameter(const std::string &name, ParameterReturnValue  parameter);

	//returns the jointID of the hinge.
	dJointID getJoint();

	unsigned int getNumActuators();

	dJointID getHinge(unsigned int index);

	//! Setting a velocity.
	/*! Motor tries to reach this velocity as fast as possible, but is constrained
	 * by maxTorque
	 */
	void setVel(float v, float deltaTime);

	//! Setting a torque directly to a joint, can't be greater than maxTorque.
	void setTorque(float torque, float deltaTime);

	//! Returns the current velocity of the joint.
	float getVel(float deltaTime);

	//! Returns the current position of the joint.
	float getPos();

	//! activate/deactivate the actuator
	void setActive(bool activate);

	//! will be called in the TickLocal function of the robotActorBase class
	void update(float deltaSimTime);

private:
	std::vector<dtCore::RefPtr<ActuatorBase> > myActuators;


};
}
#endif /* MULTIACTUATOR_H_ */
