/*
 * ScrewDrive.h
 *
 *  Created on: Nov 18, 2008
 *      Author: winkler
 */

#ifndef SCREWDRIVE2_H_
#define SCREWDRIVE2_H_

// Robot3D files
#include <srCore/actuator/actuatorBase.h>
#include <srCore/simulationUtils.h>

#ifdef AUDIO_ON
	#include <dtAudio/sound.h>
#endif

//TODO: remodel the Screw Drive according to the test results of the real one (which needs to be done as well)
//TODO: provide current screw drive information such as velocity and acceleration. Currently only the values which have been set are returned


namespace srCore {


class ScrewDrive2: public ActuatorBase {

public:
	ScrewDrive2(const std::string name);

	virtual ~ScrewDrive2();

	void registerActuator();

	void init(dtCore::RefPtr<BodyBase> attachedBody, dtCore::RefPtr<ScrewDrive2> second, bool isOnRightHandSide) {

		isOnRightHandSide = isOnRightHandSide;
		secondScrewDrive = second;
		this->attachedBody = attachedBody;

		velocity = 0;
		
#ifdef	AUDIO_ON
		screwSound = NULL;

		for(unsigned int i = 0; i < this->GetNumChildren(); ++i)
			if((screwSound = dynamic_cast<dtAudio::Sound*>(this->GetChild(i))) != NULL)
				break;
#endif
	};

	inline dtCore::RefPtr<BodyBase> getAttachedBody() {return attachedBody;};


	/**
	 * \brief  Writes the parameter value in parameter
	 *
	 *		The available parameters can be retrieved by calling getParameterList()
	 * 		See also: ObjectBase::getParameter()
	 *
	 *
	 */
	inline virtual void getParameter(const std::string &name, ParameterReturnValue  parameter) {

		//frontActuator->getParameter(name, parameter);

	}

	/**
	 * \brief  Sets a parameter
	 *
	 *		See also ObjectBase::getParameter()
	 *
	 */
	inline virtual void setParameter(const std::string &name, ParameterReturnValue  parameter) {

		//frontActuator->setParameter(name, parameter);

	}

	//returns the jointID of the hinge.
	inline dJointID getJoint() {printf("Warning: This screw drive has actually no ode hinge \n"); return 0;}
	//inline dJointID getJoint(int index) {return ((index==0) ? frontActuator->getJoint() : backActuator->getJoint());}

	//NULL;};

	// Maximum torque that can be applied by the motor.
	//virtual void setMaxTorque(float torque) = 0;

	// Maximum torque that the motor can stand(then switched off) to hold its position
	//virtual void setHoldingTorque(float torque) = 0;

	// Torque, that applies against the velocity vector, to slow down the joint.
	//virtual void setBrakeTorque(float torque) = 0;

	// Maximum speed the joint can reach.
	//virtual void setMaxSpeed(float v_max) = 0;

	//! Setting a torque directly to a joint, can't be greater than maxTorque.
	inline void setTorque(float torque, float deltaTime) {};

	//! Setting a velocity.
	/*! Motor tries to reach this velocity as fast as possible, but is constrained
	 * by maxTorque
	 */
	void setVel(float v, float deltaTime);

	//! Returns the current velocity of the joint.
	inline float getVel(float deltaTime) { return velocity; };

	//! Returns the current position of the joint.
	inline float getPos() {return 0;};

	//! Sets ode parameters for the joint. For more Information, see www.ode.org/ode-latest-user-guide.org.
	inline void setODEParameter(int parameter, float value) {};
	inline float getODEParameter(int parameter) { return 0.0; }; //Anne

	//! activate/deactivate the actuator
	inline void setActive(bool activate) {active = activate;}

	//! will be called in the TickLocal function of the robotActorBase class
	inline void update(float deltaSimTime) {setVel(getVel(deltaSimTime), deltaSimTime);}

	dtCore::RefPtr<ScrewDrive2> secondScrewDrive;
	
#ifdef	AUDIO_ON
	dtAudio::Sound* screwSound;
#endif
private:

	dtCore::RefPtr<BodyBase> attachedBody;
	bool isOnRightHandSide;

private:
  
	float d_m, my_Screw, l, F_div_M;


};
}

#endif /* SCREWDRIVE2_H_ */
