//! Specialized body class.

/*!
 * Predefined body class. Ready to use in a RobotActor.
 * This body represents the wheels of a	 robot.
 *
 * @author Lutz Winkler
 *
 */

#ifndef LED_H_
#define LED_H_

#include <srCore/robot/objectBase.h>


namespace srCore {

class ColorVisitor : public osg::NodeVisitor
{
public:

   ColorVisitor (const osg::Vec4& pColor): osg::NodeVisitor(TRAVERSE_ALL_CHILDREN), mColor(pColor)
   {

   }

   virtual void apply(osg::Geode& geode)
   {

//      osg::StateSet* ss = geode.getOrCreateStateSet();

      unsigned pNumDrawables = geode.getNumDrawables();
      for(unsigned i = 0; i < pNumDrawables; ++i)
      {
         osg::Drawable* draw = geode.getDrawable(i);
         osg::Material* mat = dynamic_cast<osg::Material*>(draw->getOrCreateStateSet()->getAttribute(osg::StateAttribute::MATERIAL));
         if(mat)
         {
            mat->setDiffuse(osg::Material::FRONT_AND_BACK, mColor);
            mat->setEmission(osg::Material::FRONT_AND_BACK, mColor);
         }
      }

   }


private:

   osg::Vec4 mColor;
};

class Led : public ObjectBase
{
public:
	//! Constructor
	Led(	dtCore::RefPtr<BodyBase> attachedBody,
			osg::Vec3f translation, osg::Vec3f orientation,
			std::string name = "noName", std::string type = "led",
			bool active = true);

	//! Copy constructor (we have dynamically allocated fields)
	Led(const srCore::Led &led);

	//! sets the position of the Body. Only use that while physics is deactivated.
	void setTranslation(osg::Vec3f pos) {};

	//! sets the rotation of the Body. Only use that while physics is deactivated.
	void setRotation(osg::Vec3f hrp) {};

	//! Sets color
	void setColor(osg::Vec4 color);

	//! Gets color
	osg::Vec4* getColor();

	//! Gets size of the LED
	float getSize();
protected:
	//! Destructor (we have dynamically allocated fields)
	virtual ~Led();

	//! Assignment operator (we have dynamically allocated fields)
	srCore::Led& operator=(const srCore::Led& led);
private:
	//! The color of the LED
	osg::Vec4* myColor;

	//! The color visitor
	ColorVisitor* cv;

	//! The LED size
	float ledSize;
};

}

#endif /*LED_H_*/
