//! Specialized body class.

/*!
 * Predefined body class. Ready to use in a RobotActor.
 * This body represents one of two parts of the current robot of UniKarl.
 *
 * @author Lutz Winkler
 *
 */

#ifndef SYMBRIONBODY01_H_
#define SYMBRIONBODY01_H_

#include <srCore/body/cuboid.h>

#include <dtAudio/sound.h>
#include <dtAudio/audiomanager.h>

namespace srCore {

class RobotActorBase;

class SymbrionBodyI : public Cuboid
{
public:

	//! Constructor
	/*! @param id 						a 16bit id, used to set the colliding bit parameter.
	 * 	@param x, y, z 					the position of the hinge in the world coordinate system.
	 *  @param heading, pitch, roll 	the orientation of the axis of the hinge in the world coordinate system.
	 *  @param Scene 					the scene in which the body should be placed.
	 *  @param osgGroup					using this group, another scene will be created that will be used by sensors
	 *
	 */
	SymbrionBodyI(	RobotActorBase* robot,
				float x, float y, float z,
				float heading, float pitch, float roll);

protected:
	virtual ~SymbrionBodyI();

	//std::queue<dtAudio::Sound*> mQueued;
//private:
	//moved to the actuators section:
//	dtAudio::Sound* bodySound;

};
}

#endif /*SYMBRIONBODY01_H_*/
