/* -*-c++-*-
 * Delta3D Open Source Game and Simulation Engine
 * Copyright (C) 2005, BMH Associates, Inc.
 *
 * Camera Component
 *
 * @author Adrian Friebel
 */

#ifndef SIMULATIONCAMERACONTROLLER_H_
#define SIMULATIONCAMERACONTROLLER_H_

// Delta3D files
#include <dtGame/gmcomponent.h>
#include <dtDAL/gameeventmanager.h>

// Robot3D files
#include <srCore/export.h>
#include <srCore/robotSelectionComponent.h>
#include <srCore/arenaComponent.h>

// TODO:	Different camera control modes, such as walking mode, automated mode, ...

/**
 * The SimulationCameraController is the camera controlled by the user to go through the
 * Robot3D simulator. It is not the one of the cameras defined on the robot.
 */
class ROBOT_EXPORT SimulationCameraController : public dtGame::GMComponent
{
public:
	SimulationCameraController(const std::string &name, osg::Vec3f camPos, bool camState);

	void ProcessMessage(const dtGame::Message &message);

	void init(dtCore::Mouse *mouse, dtCore::Keyboard *keyboard);

	void setCamera();

	//! Toggle screenshot taking (by keyboard input, default "p")
	void toggleScreenShots(bool toggle);

	void setViewport(bool fullwindow);

	//! Take screenshot and write to file
	void TakeScreenShot();

	//! Path where screenshot will be stored
	void setScreenshotPath(std::string &path);

	//! Retrieve screenshot path
	std::string getScreenshotPath();
protected:
	virtual ~SimulationCameraController();

private:
	//! Disable copying
	SimulationCameraController(const SimulationCameraController& comp);

	//! Disable copying by assignment
	void operator=(const SimulationCameraController& comp);

	//! Send message from this camera component to all other components
	void sendMessageToAll(dtDAL::GameEvent &event);

	osg::Vec3f camPos;
	bool freeCamState;

	int counter;
    srCore::ScreenShotCallback* mScreenShotTaker;

    //! Screenshot is not taken every tick... it is toggled so many ticks
	bool screenShotToggle;

	dtCore::FlyMotionModel *mFMM;
	dtCore::Tripod *mTrp;

	std::string screenShotPath;
};

#endif /*SIMULATIONCAMERACONTROLLER_H_*/
