/* -*-c++-*-
 * Delta3D Open Source Game and Simulation Engine
 * Copyright (C) 2005, BMH Associates, Inc.
 *
 * Input Component
 *
 * @author Lutz Winkler
 * @author Adrian Friebel
 */
#ifndef SIMULATIONINPUTCOMPONENT
#define SIMULATIONINPUTCOMPONENT

#include <srCore/export.h>
#include <srCore/arenaComponent.h>
#include <srCore/robotSelectionComponent.h>
#include <srCore/robot/robotActorBase.h>
#include <srCore/sensor/cameraSensorBase.h>

#include <dtGame/baseinputcomponent.h>
#include <dtGame/gameapplication.h>

#include <dtDAL/gameeventmanager.h>

#include <osgViewer/CompositeViewer>

// TODO: config file where the keyboard layout is saved

class ROBOT_EXPORT SimulationInputComponent : public dtGame::BaseInputComponent
{
public:
	SimulationInputComponent(const std::string &name, bool robCtrlEna);
	void ProcessMessage(const dtGame::Message &message);

	bool HandleKeyPressed(const dtCore::Keyboard* keyboard,int key);
	bool HandleButtonPressed(const dtCore::Mouse* mouse, dtCore::Mouse::MouseButton button);

	void PrintPossibilities();
protected:
	virtual ~SimulationInputComponent();

private:
	//! Disable copying
	SimulationInputComponent(const SimulationInputComponent& comp);

	//! Disable copying by assignment
	void operator=(const SimulationInputComponent& comp);

	void sendMessageToAll(dtDAL::GameEvent &event);
	void sendMessageToRobot(dtDAL::GameEvent &event);

	void commMessage();

	dtCore::RefPtr<dtDAL::GameEvent> commMsg;

	//Controlling the robot
	dtCore::RefPtr<dtDAL::GameEvent> mAccelerate;
	dtCore::RefPtr<dtDAL::GameEvent> mDecelerate;
	dtCore::RefPtr<dtDAL::GameEvent> mMoveActorUp;
	dtCore::RefPtr<dtDAL::GameEvent> mMoveActorDown;
	dtCore::RefPtr<dtDAL::GameEvent> mMoveActorLeft;
	dtCore::RefPtr<dtDAL::GameEvent> mMoveActorRight;

	//Environmental controls
	dtCore::RefPtr<dtDAL::GameEvent> mSavingPicture;
	dtCore::RefPtr<dtDAL::GameEvent> mSaveScreenShot;

	//Camera Controls
	dtCore::RefPtr<dtDAL::GameEvent> mZoomOut;
	dtCore::RefPtr<dtDAL::GameEvent> mZoomIn;
	dtCore::RefPtr<dtDAL::GameEvent> mConnectAll;
	dtCore::RefPtr<dtDAL::GameEvent> mDisconnectAll;
	dtCore::RefPtr<dtDAL::GameEvent> mCamMoveLeft;
	dtCore::RefPtr<dtDAL::GameEvent> mCamMoveRight;
	dtCore::RefPtr<dtDAL::GameEvent> mCamMoveUp;
	dtCore::RefPtr<dtDAL::GameEvent> mCamMoveDown;

	dtCore::RefPtr<dtDAL::GameEvent> mTimeScaleChange;
	dtCore::RefPtr<dtDAL::GameEvent> mSelectRobot;
	dtCore::RefPtr<dtDAL::GameEvent> mMouseLBClick;
	dtCore::RefPtr<dtDAL::GameEvent> mSwitchControl;

	dtCore::RefPtr<dtDAL::GameEvent> mPauseSim;
	dtCore::RefPtr<dtDAL::GameEvent> mResetSim;
	dtCore::RefPtr<dtDAL::GameEvent> mDestroyAllActor;
	dtCore::RefPtr<dtDAL::GameEvent> mDestroyActor;
	dtCore::RefPtr<dtDAL::GameEvent> mCreateActor;

	// Controller Controls
#ifdef PARTNER_ALMENDE
dtCore::RefPtr<dtDAL::GameEvent> mToggleLearnToDrive;
dtCore::RefPtr<dtDAL::GameEvent> mToggleLearnBinLoc;
dtCore::RefPtr<dtDAL::GameEvent> mToggleImageSaliency;
dtCore::RefPtr<dtDAL::GameEvent> mToggleAudioSaliency;
#endif

bool takeScreenshot;
bool rotAroundRobot;
bool robotControlEnabled;

//      int counter;
int ticks;

//      srCore::ScreenShotCallback* mScreenShotTaker;
};

#endif

