/**
 * @file loadComponents.h
 *
 * This file loadComponents.h is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license,
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Aug 4, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	Plugins
 */

#ifndef LOADCOMPONENTS_H_
#define LOADCOMPONENTS_H_

// General files
#include <vector>

// Open Scene Graph
#include <osg/Referenced>

// Boost library
#include <boost/filesystem.hpp>

// Delta3D files
#include <dtGame/gameapplication.h>

namespace srCore {

/* **************************************************************************************
 * Interface of loadComponents
 * **************************************************************************************/


class LoadComponents: public osg::Referenced {
public:
	/**
	 * Constructor loadComponents
	 */
	LoadComponents();

	/**
	 * Destructor ~loadComponents
	 */
	virtual ~LoadComponents();

	/**
	 *
	 */
	void LoadLibs(dtGame::GameApplication &application);

	/**
	 *
	 */
	void SetLibPath(const std::string &libpath);

	/**
	 * Get all .so files
	 */
	void GetLibraries();

	/**
	 * Prints a list of the components that are loaded.
	 */
	void PrintAllComponents();
protected:

private:
	boost::filesystem::path path;

	std::vector<std::string> library;

	bool initialised;

	bool error;

	std::vector<void*> handles;
};

}

#endif /* LOADCOMPONENTS_H_ */
