/**
 * @file Robot3DComponent.h
 * @brief 
 *
 * This file is created at Almende B.V. It is open-source software and part of the Common 
 * Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools, ranging from 
 * thread pools and TCP/IP components to control architectures and learning algorithms. 
 * This software is published under the GNU Lesser General Public license (LGPL).
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Nov 15, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	
 */


#ifndef ROBOT3DCOMPONENT_H_
#define ROBOT3DCOMPONENT_H_

// General files

// Delta3D files
#include <dtGame/gmcomponent.h>

// Robot3D files
#include <srCore/xmlConfig.h>

namespace srCore {

/* **************************************************************************************
 * Interface of Robot3DComponent
 * **************************************************************************************/


class Robot3DComponent: public dtGame::GMComponent {
public:

	//! Name which can be used in config.xml file
	static const std::string DEFAULT_NAME;

	/**
	 * Constructor Robot3DComponent
	 */
	Robot3DComponent(const std::string &name);

	/**
	 * Destructor ~Robot3DComponent
	 */
	virtual ~Robot3DComponent();

	//! Default process message routine
	void ProcessMessage(const dtGame::Message &message);

	//! Return configuration object
	GetConfig& GetConfigObject();
protected:

private:
	//! Disable copying
	Robot3DComponent(const Robot3DComponent& comp);

	//! Disable copying by assignment
	void operator=(const Robot3DComponent& comp);

	// Configuration information
	GetConfig appConfig;

};

}

#endif /* ROBOT3DCOMPONENT_H_ */
