//! Camera sensor class.

/*!
 *
 * Describes general camera properties.
 *
 * @author Lutz Winkler
 *
 */

#ifndef CAMERASENSOR_H_
#define CAMERASENSOR_H_

#include <srCore/body/bodyBase.h>
#include <srCore/sensor/cameraSensorBase.h>

namespace srCore {

/**
 * The CameraSensor does not so much as populating the CameraSensorBase class with
 * default values that make sense for an actual camera. The resolution is set to
 * 256x256 pixels.
 */
class CameraSensor: public CameraSensorBase {
public:

	/**
	 * \brief  Initializes the sensor
	 *
	 * \param			gm							GameManager, which handles the corresponding robot
	 * \param			attachedTo					Body, where the camera is attached to
	 * \param     		pos			   				Local position of the camera
	 * \param			orient 						Local orientation of the camera
	 * \param			flareAngle					flare angle of the camera
	 * \param			resolution					resolution of the camera
	 * \param			name						name of the camera
	 */
	CameraSensor(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3 pos, osg::Vec3 orient,
			float flareAngle = 1,
			osg::Vec2 resolution = osg::Vec2(256,256),
			const std::string &name="CameraSensor");

	//! Destructor
	virtual ~CameraSensor();

	//! Sends command to retrieve data
	void evaluate(ParameterReturnValue srv);

	//! Does not return new camera image but updates internal field
	inline void getNewSensorValue() {requestData();}

private:
	//! Internal storage of image value
	osg::ref_ptr<osg::Image> value;
protected:
	//! Retrieve image by stream operator
	inline void getSensorValue(std::ostream& theStream) const { theStream << value;}

};

}

#endif /*CAMERASENSOR_H_*/
