/*
 * depthImageCamera.h
 *
 *  Created on: Dec 29, 2008
 *      Author: winkler
 */

#ifndef DEPTHIMAGECAMERA_H_
#define DEPTHIMAGECAMERA_H_


#include "cameraSensorBase.h"
#include "../bodies/bodyBase.h"

namespace srCore {

class DepthImageCamera: public CameraSensorBase
{
public:

	/**
	 * \brief  Initializes the sensor
	 *
	 * \param			gm							GameManager, which handles the corresponding robot
	 * \param			attachedTo					Body, where the camera is attached to
	 * \param     		pos			   				Local position of the camera
	 * \param			orient 						Local orientation of the camera
	 * \param			flareAngle					flare angle of the camera
	 * \param			resolution					resolution of the camera
	 * \param			name						name of the camera
	 *
	 *
	 */
	DepthImageCamera(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3 pos, osg::Vec3 orient, float flareAngle = 1, osg::Vec2 resolution = osg::Vec2(256,256), const std::string &name="CameraSensor");
	virtual ~DepthImageCamera() { value=NULL; }

	//! sends command to retrieve data
	void evaluate(ParameterReturnValue srv);
	inline void getNewSensorValue() {requestData();}
private:
	//osg::ref_ptr<osg::Image> value;
	osg::Image* value;
protected:
	inline void getSensorValue(std::ostream& theStream) const { theStream << value;}

};

}

#endif /* DEPTHIMAGECAMERA_H_ */
