//! Laser Scanner class.

/*!
 *
 * Describes general laser scanner properties.
 *
 * @author Lutz Winkler
 *
 */

#ifndef LASERSCANNER_H_
#define LASERSCANNER_H_

#include "raySensorBase.h"

namespace srCore {

class LaserScanner : public RaySensorBase
{
public:

	/**
	 * \brief  Initializes the sensor
	 *
	 * \param			gm							GameManager, which handles the corresponding robot
	 * \param			attachedTo					Body, where the scanner is attached to
	 * \param     		pos			   				Local position of the scanner
	 * \param			orient 						Local orientation of the scanner
	 * \param			flareAngle					flare angle of the scanner
	 * \param			resolution					resolution of the scanner
	 * \param			name						name of the scanner
	 *
	 *
	 */
	LaserScanner(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3f pos, osg::Vec3f orient, float flareAngle, osg::Vec2 resolution, const std::string &name);
	virtual ~LaserScanner();

	//! sends command to retrieve data
	void evaluate(ParameterReturnValue srv);

	void getSensorValue(std::ostream& theStream) const {theStream << currentSensorValue;};

//	void update() {RaySensorBase::update();};

private:
	osg::ref_ptr<osg::Image> value;

};

}

#endif /*LASERSCANNER_H_*/
