//! Light Sensor class.

/*!
 *
 * Describes general light sensor properties.
 *
 * @author Lutz Winkler
 *
 */

#ifndef LIGHTSENSOR_H_
#define LIGHTSENSOR_H_

#include "cameraSensorBase.h"

namespace srCore {



class LightSensor: public CameraSensorBase
{
public:

	/**
		 * \brief  Initializes the sensor
		 *
		 * \param			gm							GameManager, which handles the corresponding robot
		 * \param			attachedTo					Body, where the sensor is attached to
		 * \param     		pos			   				Local position of the sensor
		 * \param			orient 						Local orientation of the sensor
		 * \param			flareAngle					flare angle of the sensor
		 * \param			name						name of the sensor
		 *
		 *
		 */
	LightSensor(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3 pos, osg::Vec3 orient, float flareAngle, const std::string &name="LightSensor");
	virtual ~LightSensor();
	//const std::string TakeScreenShot(const std::string& namePrefix);

	/**
	 * \brief	Calculates a new sensor value
	 *
	 * It will be called every time step if it is an active sensor, otherwise it needs to be called by the controller
	 *
	 */
	virtual void getNewSensorValue();

	//! sends command to retrieve data
	void evaluate(ParameterReturnValue srv);

protected:
	virtual void getSensorValue(std::ostream& theStream) const;


private:
	unsigned int value;

};
}

#endif /*LIGHTSENSOR_H_*/
