/* -*-c++-*-
 * Delta3D Open Source Game and Simulation Engine
 * Copyright (C) 2006, Alion Science and Technology, BMH Operation
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * @author Curtiss Murphy
 */
#ifndef SIMULATION_ENTRY_POINT_H_
#define SIMULATION_ENTRY_POINT_H_

#include <osg/Texture2D>

#include <osg/Material>
#include <osg/ShapeDrawable>
#include <osgGA/StateSetManipulator>

#include <osgViewer/Viewer>
#include <osgViewer/ViewerEventHandlers>



#include <osgDB/ReadFile>
#include <osg/LightSource>

#define DELTA3D_2_3

#include <osg/Notify>
#include <osg/ComputeBoundsVisitor>
#include <osg/PolygonOffset>
#include <osg/CullFace>
#include <osg/io_utils>

#include <dtABC/application.h>

#include <dtCore/camera.h>
#include <dtCore/flymotionmodel.h>
#include <dtCore/keyboard.h>
#include <dtCore/scene.h>
#include <dtCore/tripod.h>
#include <dtCore/uniqueid.h>


#include <dtCore/databasepager.h>



#include <dtDAL/project.h>
#include <dtDAL/map.h>

#include <dtGame/gamemanager.h>
#include <dtGame/defaultmessageprocessor.h>
#include <dtGame/gameapplication.h>

#include <dtABC/proximitytrigger.h>
#include <dtCore/refptr.h>
#include <dtGame/gameentrypoint.h>


#include <iostream>

#include <srCore/export.h>
#include <srCore/xmlConfig.h>
#include <srCore/simulationUtils.h>
#include <srCore/arenaComponent.h>
#include <srCore/robotSelectionComponent.h>
#include <srCore/cameraComponent.h>
#include <srCore/inputComponent.h>
#include <srCore/guiComponent.h>
#include <srCore/loadComponents.h>


#ifdef DELTA3D_2_3
	#include <dtCore/databasepager.h>
#endif
#ifdef AUDIO_ON
#include <dtAudio/audiomanager.h>
#endif


// TODO: console output triggered by some kind of message level (i.e.: NONE, ERROR, WARNING, INFO, ALL)
// TODO: clean up the entry point. Only components, controllers and actors should be initialized here

///The user data associated with "collision" messages
struct ROBOT_EXPORT CollisionData
{
	dtCore::Transformable* mBodies[2]; ///<The bodies colliding
	osg::Vec3 mLocation; ///<The collision location
	osg::Vec3 mNormal; ///<The collision normal
	float mDepth; ///<The penetration depth
};

namespace dtGame
{
   class GameApplication;
   class GameManager;
   class GameActorProxy;
}

/*!
 * Our entry point into the simulation.  The GameStart.exe application can load this like this:
 *     "GameStartd.exe TutorialGameActors"
 * We create our Game Manager and components in this class.
 */
class ROBOT_EXPORT SimulationEntryPoint: public dtGame::GameEntryPoint
{
public:

	/// Default constructor
	SimulationEntryPoint();

	/// Default destructor
	virtual ~SimulationEntryPoint();

	/**
	 * Called just before your application's game loop starts.  This is your main
	 * opportunity to create components, load maps, create unique actors, etc...
	 */
	virtual void OnStartup(dtGame::GameApplication &app);

	/**
	 * Set all the physical properties. Currently the ODE simulator is used, but it is possible
	 * (in theory!) to use the Delta3D dtPhysics engine that is build on top of PAL, the
	 * Physics Abstraction Layer that can use
	 */
	void SetPhysicalParameters(dtGame::GameApplication &app);

	/**
	 * Set parameters for the GUI renderer (CEGUI path and filenames)
	 */
	void InitGUI(dtGame::GameApplication &app);

	/**
	 * The GUI components are handled in this separate routine. It needs some care to make sure
	 * that the cameras on the robots don't see the GUI components for example.
	 */
	void LoadGUIComponents(dtGame::GameApplication &app);
protected:

	/**
	 * Add the camera component to the scene
	 */
	void AddCamera();
private:

	dtCore::RefPtr<srCore::SimulationGUIComponent> guiComp;

	dtCore::RefPtr<srCore::LoadComponents> loadComp;
};
#endif
