/**
 * @file singleton.h
 * @brief 
 *
 * This file singleton.h is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license,
 * but the software is not allowed to be used for military use, within the bio-industry
 * or for animal experimentation.
 *
 * @author  Anne C. van Rossum
 * @date    Jul 13, 2010
 * @project Replicator FP7
 * @company Almende B.V.
 * @case	
 */


#ifndef SINGLETON_H_
#define SINGLETON_H_

// General files
#include <cassert>

namespace srCore {

/**
 * \addtogroup Default
 * @{
 */

/* **************************************************************************************
 * Interface of singleton
 * **************************************************************************************/


template <class T>
class Singleton
{
public:
	static T* Instance() {
		if(!m_pInstance) m_pInstance = new T;
		assert(m_pInstance !=NULL);
		return m_pInstance;
	}
protected:
	Singleton();
	~Singleton();
private:
	Singleton(Singleton const& singleton);
	Singleton& operator=(Singleton const& singleton);
	static T* m_pInstance;
};

template <class T> T* Singleton<T>::m_pInstance=NULL;


///@}

}

#endif /* SINGLETON_H_ */
