/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */

#ifndef ACTUATORFACTORY_H_
#define ACTUATORFACTORY_H_

#include <srActuator/screwDrive.h>
#include <srActuator/HingeUniKarl.h>
#include <srActuator/WheelUniKarl.h>
#include <srActuator/multiActuator.h>

/** srCore namespace
 *
 * Currently, in that namespace, the factories and their type enumerations are included.
 *
 */
namespace srCore {


/**
 * Actuator Type Enumeration
 */
enum actuatortype
{
	HINGE_ACTUATOR,
	WHEEL_ACTUATOR,

};


//! Actuator Factory Class
class ActuatorFactory	// : public myFactory
{
public:

	//! Creates an actuator
		/*! @param type								type of the actuator
		 *  @param body1, body2 					the bodies the hinge is attached to.
		 * 	@param anchorX, anchorY, anchorZ 		the position of the hinge in the world coordinate system.
		 *  @param axisX, axisY, axisZ 				the orientation of the axis of the hinge in the world coordinate system.
		 * 	@param gm								the GameManager that handles the corresponding robot
		 *
		 */
	static dtCore::RefPtr<ActuatorBase> createActuator(
			actuatortype type,
			dtCore::RefPtr<BodyBase> myBody1, dtCore::RefPtr<BodyBase> myBody2,
			float anchorX, float anchorY, float anchorZ,
			float axisX, float axisY, float axisZ, const std::string &name);

	virtual ~ActuatorFactory();

};

}		/*namespace*/

#endif /*ACTUATORFACTORY_H_*/
