#ifndef SENSORFACTORY_H_
#define SENSORFACTORY_H_

#include <dtGame/gamemanager.h>

#include <srCore/sensor/cameraSensor.h>

namespace srCore {

/**
 * Sensor Type Enumeration
 */
enum SensorType
{
	CAMERA_SENSOR,
//	LIGHT_SENSOR,
//	LASER_SCANNER,
//	DISTANCE_SENSOR,
//	DEPTH_IMAGE_SENSOR,
//	SAMPLE_SENSOR,
//	LUMINANCE_SENSOR,
//	MICROPHONE_SENSOR,
//	INFRARED_SENSOR,
//	DOCKING_SENSOR,
	UNKOWN_SENSOR
};

/**
 *  Sensor Factory Class
 */
class SensorFactory	// : public myFactory
{
public:
	/**
	 * \brief Takes name of a sensor type and returns matching SensorType
	 *
	 * \param 	name 	name of sensor type
	 *
	 * \return SensorType
	 */
	static SensorType getSensorTypeByName(const char* name);

	/**
	 * \brief  Creates a sensor
	 *
	 * \param			type						the sensor type
	 * \param			attachedTo					Body, where the sensor is attached to
	 * \param     		anchorX, anchorY, anchorZ	Local position of the sensor
	 * \param			axisX, axisY, axisZ			Look-at vector of the sensor
	 * \param			gm							GameManager, which handles the corresponding robot
	 * \param			name						name of the sensor
	 *
	 *
	 */
	static dtCore::RefPtr<SensorBase> createSensor(
			SensorType type,
			const std::string &name,
			dtCore::RefPtr<BodyBase> attachedBody,									// body where the sensor is attached to
			osg::Vec3f position, 									// relative position of the sensor to the attached object
			osg::Vec3f orientation,									// vector where the sensor is aimed to
			float apertureAngle = 1.0,
			osg::Vec2 resolution = osg::Vec2(256,256));

	virtual ~SensorFactory();

};

}		/*namespace*/


#endif /*SENSORFACTORY_H_*/
