/**
 * @file Robot3DVocab.h
 * @brief 
 *
 * This file is created at Almende B.V. It is open-source software and part of the Common 
 * Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools, ranging from 
 * thread pools and TCP/IP components to control architectures and learning algorithms. 
 * This software is published under the GNU Lesser General Public license (LGPL).
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Nov 19, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	
 */


#ifndef ROBOT3DVOCAB_H_
#define ROBOT3DVOCAB_H_

// General files
#include <vector>
#include <string>

// Yarp files
#include <yarp/os/Bottle.h>
#include <yarp/os/Vocab.h>

// Plugin files
#include <srCore/singleton.h>

using namespace yarp::os;

namespace srInterface {

/*
 * Robot sides
 *
 * There are four sides, named after the wind rose directions. It is not intended to
 * predefine if this actually means those absolute directions with respect to an
 * external magnetic field for example.
 */
#define VOCAB_FRONT VOCAB4('f','r','n','t')
#define VOCAB_BACK  VOCAB4('b','a','c','k')
#define VOCAB_LEFT  VOCAB4('l','e','f','t')
#define VOCAB_RIGHT VOCAB4('r','g','h','t')
#define VOCAB_NONE  VOCAB4('n','o','n','e')

/*
 * Add and remove robots
 */
#define VOCAB_ROBOT_ADD  VOCAB4('r','o','b','+')
#define VOCAB_ROBOT_DEL  VOCAB4('r','o','b','-')

/*
 * Commands to the actuators (the wheels)
 */
#define VOCAB_WHEEL_LEFT	VOCAB('w','h','l','l')
#define VOCAB_WHEEL_RIGHT	VOCAB('w','h','l','r')

/**
 * RPC commands
 */
#define RPC_VOCAB_OK		VOCAB2('o','k')				// Okay
#define RPC_VOCAB_FAILED 	VOCAB4('f','a','i','l')		// Fail
#define RPC_VOCAB_PORT 		VOCAB4('p','o','r','t')		// Port information

#define RPC_VOCAB_CCAM		VOCAB4('c','c','a','m')		// Configure camera

//! See <yarp/dev/ControlBoardInterfaces.h>

#undef RPC_VOCAB_FAILED
#undef RPC_VOCAB_OK

/* **************************************************************************************
 * Interface of Robot3DVocab
 * **************************************************************************************/


class Robot3DVocab_ {
public:
	/**
	 * Constructor Robot3DVocab
	 */
	Robot3DVocab_();

	/**
	 * Destructor ~Robot3DVocab
	 */
	virtual ~Robot3DVocab_();

	//! Add vector of strings to bottle
	void toBottle(const std::vector<std::string> &str, Bottle &result);

	//! Add string to bottle
	void toBottle(const std::string &str, Bottle &result);

protected:

private:

};

}

/**
 * Robot3DVocab, this is the class definition that can be used. Use it by for example
 * Robot3DVocab::Instance().StringToChar.
 */
typedef srCore::Singleton<srInterface::Robot3DVocab_> Robot3DVocab;

#endif /* ROBOT3DVOCAB_H_ */
