/**
 * @file YarpPlugin.h
 *
 * This file YarpPlugin.h is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license,
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * @author 	Anne C. van Rossum
 * @date	Aug 4, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	
 */


#ifndef YARPPLUGIN_H_
#define YARPPLUGIN_H_

// General files

// Robot3D files
#include <srCore/pluginEntryPointBase.h>
#include <srCore/singleton.h>
#include <srInterface/YarpedSim.h>

namespace srInterface {

/* **************************************************************************************
 * Interface of YarpPlugin
 * **************************************************************************************/


class YarpPlugin: public srCore::PluginEntryPointBase {
public:
	/**
	 * Constructor YarpPlugin
	 */
	YarpPlugin();

	/**
	 * Destructor ~YarpPlugin
	 */
	virtual ~YarpPlugin();

	/// The to-be-overloaded function called from Robot3D
	virtual void StartPlugin(dtGame::GameApplication &application);

protected:

private:
	YarpedSim *yarped_sim;

	//! Forbid copying by copy constructor
	YarpPlugin(const YarpPlugin& plugin);

	//! Forbid copying by assignment
	YarpPlugin& operator=(const YarpPlugin& plugin);

};

}

/**
 * YarpPluginSingleton, this is the class definition that can be used. Use it by
 * YarpPluginSingleton::Instance().
 */
typedef srCore::Singleton<srInterface::YarpPlugin> YarpPluginSingleton;


#endif /* YARPPLUGIN_H_ */
