/**
 * @file YARPUtil.h
 *
 * This file YARPUtil.h is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license,
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Aug 3, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	YARP
 */

#ifndef YARPUTIL_H_
#define YARPUTIL_H_

// General files
#include <string>

// RobotD files
#include <srCore/singleton.h>
#include <srCore/parameterReturnValue.h>

// Yarp files
#include <yarp/os/Value.h>
#include <yarp/os/Vocab.h>
#include <yarp/os/Bottle.h>

using namespace yarp::os;

namespace srInterface {

/* **************************************************************************************
 * Interface of YarpUtil
 * **************************************************************************************/

/**
 * YarpUtil_, do not use this class directly. Use the singleton version that does not end in
 * an underscore. So, use YarpUtil::Instance()->StringToChar e.g.
 */
class YarpUtil_ {
public:

	/**
	 * Constructor of YarpUtil_
	 */
	YarpUtil_();

	/**
	 * Destructor of YarpUtil_
	 */
	virtual ~YarpUtil_();

	/**
	 * Convert string to char pointer. Be careful with this, you can work around
	 * const strings and chars in this way, but it is not for nothing that "const"
	 * exists.
	 */
	char *StringToChar(std::string str);

	/**
	 * The bottle->pop() routine doesn't work properly. Hence, we temporarily replace it with
	 * something that works, but we need to copy everything in and out...
	 */
	Value * pop_front(Bottle &bottle);

	/**
	 * Translate a bottle to a ParameterReturnValue.
	 * @param 	offset	Adds an offset, such that the first part of the bottle isn't read
	 */
	srCore::ParameterReturnValue * BottletoPRV(const Bottle &bottle, int offset = 0);

	/**
	 * Translate a ParameterReturnValue to a bottle
	 */
	void PRVtoBottle(const srCore::ParameterReturnValue &param, Bottle &bottle);

protected:

private:

};

}

/**
 * YarpUtil, this is the class definition that can be used. Use it by for example
 * YarpUtil::Instance().StringToChar.
 */
typedef srCore::Singleton<srInterface::YarpUtil_> YarpUtil;

#endif /* YARPUTIL_H_ */
