/*
* Delta3D Open Source Game and Simulation Engine
* Copyright (C) 2006, Alion Science and Technology, BMH Operation
*
* This library is free software; you can redistribute it and/or modify it under
* the terms of the GNU Lesser General Public License as published by the Free
* Software Foundation; either version 2.1 of the License, or (at your option)
* any later version.
*
* This library is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
* details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this library; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* @author Matthew W. Campbell
* @editor Allen M. Danklefsen
* @editor Curtiss Murphy
*/
#ifndef ROBOT_KIT2
#define ROBOT_KIT2


#include <srCore/robot/robotActorBase.h>

using namespace srCore;

namespace srRobot {

/**
 * This class is the actor that represents a robot.  It knows how to
 * respond to keyboard presses (I, J, K, L to steer). It also listens for
 * "SpeedBoost" GameEvent.  This actor correctly handles
 * TickLocal versus TickRemote.
 */
class ROBOT_EXPORT RobotKIT2 : public RobotActorBase
{
public:
	//! Constant identifier for our game event handler method.
	static const std::string EVENT_HANDLER_NAME;

	//! Name to be used in configuration file e.g.
//	static const std::string DEFAULT_NAME;

	//! Constructs the robot actor.
	RobotKIT2(dtGame::GameActorProxy &proxy, float jointAngle=0);

	/**
	 * This method is an invokable called when an object is local and
	 * receives a tick.
	 * @param tickMessage A message containing tick related information.
	 */
	virtual void TickLocal(const dtGame::Message &tickMessage);

	/**
	 * This method is an invokable called when an object is remote and
	 * receives a tick.
	 * @param tickMessage A message containing tick related information.
	 */
//	virtual void TickRemote(const dtGame::Message &tickMessage);

	/**
	 * Generic handler for messages. Overridden from base class.
	 * This is the default invokable on GameActorProxy.
	 */
	virtual void ProcessMessage(const dtGame::Message &message);

	/** Called when the actor has been added to the game manager.
	 * You can respond to OnEnteredWorld on either the proxy or actor or both.
	 */
	virtual void OnEnteredWorld();




protected:
	virtual ~RobotKIT2();

	/**
	 * build the robot from meshes and attach joints and physical features to it
	 * you can set the starting position & heading with the normal proxy
	 * SetTranslation & SetRotation method analogical (only the first value -
	 * the heading in the x-y-plane is used)
	 * this Method is called from OnEnteredWorld(), so you have to call
	 * the SetTranslation & SetRotation methods before the AddActor call
	 */
	void 		Initialize();

private:
	//! delta time between two simulation steps
	double deltaTime;

	//! switch between manual (keyboard and GUI control) and automat (Controller) control
	bool manual;


};

/**
 * Our proxy class for the robot actor.  The proxy contains properties,
 * invokables, and robot actor.
 */
class ROBOT_EXPORT RobotKIT2Proxy : public RobotActorBaseProxy//dtGame::GameActorProxy
{
public:

	//! Constructs the proxy.
	RobotKIT2Proxy();

	//! Creates the properties that are custom to the robot proxy.
	virtual void BuildPropertyMap();

protected:
	virtual ~RobotKIT2Proxy();

	//! Creates an instance of our robot tank actor
	virtual void CreateActor();

	/** Called when this proxy is added to the game manager (ie, the "world")
	 * You can respond to OnEnteredWorld on either the proxy or actor or both.
	 */
	virtual void OnEnteredWorld();
};

}

#endif			//ROBOT_KIT2
