
/*
 * Delta3D Open Source Game and Simulation Engine
 * Copyright (C) 2006, Alion Science and Technology, BMH Operation
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * @author Matthew W. Campbell
 */
#ifndef ROBOT_REGISTRY
#define ROBOT_REGISTRY

// General files
#include <iostream>

// Delta3D files
#include <dtDAL/actorpluginregistry.h>
#include <dtGame/gameapplication.h>

// Robot3D files
#include <srCore/export.h>
#include <srCore/xmlConfig.h>

namespace srRobot {

/**
 * Allows a robot to register. A robot needs to be registered before it can be loaded from e.g.
 * an XML configuration file. The ROBOT_EXPORT option makes it possible to export this library
 * and for example use it in the Delta3D STAGE editor.
 *
 * @url http://www.delta3d.org/article.php?story=20050720135643860
 */
class ROBOT_EXPORT RobotRegistry : public dtDAL::ActorPluginRegistry {
public:
	//! Constructs our registry.  Creates the actor types easy access when needed.
	RobotRegistry();

	//! Registers actor types with the actor factory in the super class.
	void RegisterActorTypes();

	//! Adds robot to the game manager
//	static void addRobotToGM(int id, RobotInitData parameters, float height);

	//! A karlsruhe robot
	static dtCore::RefPtr<dtDAL::ActorType> RobotKITType;

	//! A karlsruhe robot, with a different implementation of the screw drive
	static dtCore::RefPtr<dtDAL::ActorType> RobotKIT2Type;
private:
	//! An initialised flag (mainly for debugging purposes)
	bool initialised;
};

} // end of namespace srCore


#endif  /*ROBOT_REGISTRY*/
