/*
 * sampleSensor.h
 *
 *  Created on: Nov 28, 2008
 *      Author: winkler
 */

#ifndef SAMPLESENSOR_H_
#define SAMPLESENSOR_H_

// The following todo list do not refer to this class, but to the whole directory.
// The model/sensor directory is quite a stub. Please help to improve it ;)

/* TODO:	improved sensor model. Different kind of sensor deviations should be considered to be included in such a model, including:
 * 			- noise
 * 			- resolution
 * 			- measurement of a property other than the one the sensor has to measure (such as temperature)
 * 			- offset /bias
 * 			- drift (short term / long term)
 * 			- hysteresis
 * 			- sample rate
 */

/* TODO: 	humidity / temperature / gas dispersion sensor,
 * 			including a model that describes the dispersion of this environmental property. A special GMComponent need to be implemented,
 * 			that simulates the gas dispersion and the sensor requests the current value at its given transform.
 */

/* TODO:	When an improved sensor model has been defined and it is clear what sensors will be on the real robot, these sensors need
 * 			to be implemented accordingly.
 */

#include <srCore/sensor/sensorBase.h>

namespace srCore {

class SampleSensor : public SensorBase
{
public:
	SampleSensor(dtCore::RefPtr<BodyBase> attachedBody = NULL,
			osg::Vec3f translation = osg::Vec3f(2,0,0), osg::Vec3f orientation= osg::Vec3f(0,0,0),
			std::string name = "noName",
			bool active = true);

	virtual ~SampleSensor();

	/**
	 * \brief  Writes the parameter value in parameter
	 *
	 *		The available parameters can be retrieved by calling getParameterList()
	 *		See also: ObjectBase::getParameter()
	 *
	 *
	 */
	void getParameter(const std::string &name, ParameterReturnValue  parameter);

	/**
	 * \brief  Sets a parameter
	 *
	 *		See also getParameter()
	 *
	 */
	void setParameter(const std::string &name, ParameterReturnValue  parameter);

	//! sends command to retrieve data
	void evaluate(ParameterReturnValue srv);

	float myFancySimulatedEnvironmentFunction(osg::Vec3f pos, osg::Vec3f rot);

	/**
	 * \brief Request to get new data
	 *
	 * 		Put here your algorithms for simulated temperature, humidity and  what so ever that can be expressed by function depending on the position and orientation.
	 *
	 */
	void requestData();

	/**
	 * \brief	Calculates a new sensor value
	 *
	 * It will be called every time step if it is an active sensor, otherwise it needs to be called by the controller
	 *
	 */
	virtual void getNewSensorValue();

	//! sets up sensor
	void setup() {};

	//! Will be called in RobotActorBase::TickLocal()
	void update(float deltaSimTme);


protected:
	virtual void getSensorValue(std::ostream& theStream) const;

private:

	float sensorValue;

};
}

#endif /* SAMPLESENSOR_H_ */
