#!/bin/bash

# This file list_authors.sh is created at Almende B.V. It is open-source software and part
# of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
# Ranging from thread pools, and TCP/IP components to control architectures and learning
# algorithms. This software is published under the GNU Lesser General Public license, under
# version 3 (or later). You can find a version of the license on Debian (derived) systems at
# /usr/share/common-licenses/LGPL-3.

# It is not possible to add usage restrictions to a free open-source license. Nevertheless, we
# personally strongly object against this software used by the military, in the bio-industry, 
# for animal experimentation, or anything that violates the Universal Declaration of Human 
# Rights. This is, again, no usage restriction, it is a non-binding recommendation.

# AUTHOR © 2010 Anne van Rossum <anne@almende.com>

FILE="files_to_check.txt"

N=30

PRINT_NO_AUTHOR=

if [ $# -eq 1 ]; then
	if [ "$1" == "--without" ]; then
		echo "Print files that do not have a AUTHOR statement"
		PRINT_NO_AUTHOR="TRUE"
	else 
		echo "Print files with AUTHOR statements"
	fi
fi

rm $FILE

THIS_FILE=${0##*/}

find * | grep -v .svn | grep -v build/ | grep -v -i $THIS_FILE > $FILE

NUMBER_OF_FILES=`< $FILE wc -l`
	
echo "Check recursively in directory if there is a AUTHOR statement in the first $N lines"

for i in $(seq 1 1 $NUMBER_OF_FILES)
do
	CHECKFILE=`< $FILE head -n $i | tail -n 1`
	if [ -d $CHECKFILE ]; then
		# Is directory, don't cat the file
		#echo "Enter directory: $CHECKFILE"
		continue
	fi
	AUTHOR=`head -n $N $CHECKFILE | grep -i author | grep -v -i Winkler | grep -v Anne | grep -v Ted | grep -v Adrian | grep -v Rusu`
#	AUTHOR=`echo $AUTHOR | grep -v Winkler`
	if [ -z "$AUTHOR" ]; then
		if [ -n "$PRINT_NO_AUTHOR" ]; then
			echo "File $CHECKFILE has no AUTHOR information"
			continue
		fi 
	else 
		if [ -z "$PRINT_NO_AUTHOR" ]; then
		        echo "File $CHECKFILE contains AUTHOR: $AUTHOR"
#			echo "$AUTHOR"
		fi 
        fi
done
