#!/bin/bash

# This file check_copyright.sh is created at Almende B.V. It is open-source software and part
# of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
# Ranging from thread pools, and TCP/IP components to control architectures and learning
# algorithms. This software is published under the GNU Lesser General Public license, under
# version 3 (or later). You can find a version of the license on Debian (derived) systems at
# /usr/share/common-licenses/LGPL-3.

# It is not possible to add usage restrictions to a free open-source license. Nevertheless, we
# personally strongly object against this software used by the military, in the bio-industry, 
# for animal experimentation, or anything that violates the Universal Declaration of Human 
# Rights. This is, again, no usage restriction, it is a non-binding recommendation.

# Date 		Sep. 18, 2010
# Author	Anne van Rossum

# Copyright © 2010 Anne van Rossum <anne@almende.com>

FILE_CHECKFORCOPYRIGHT="files_to_check.txt"

N=30

PRINT_NO_COPYRIGHT=

if [ $# -eq 1 ]; then
	if [ "$1" == "--without" ]; then
		echo "Print files that do not have a copyright statement"
		PRINT_NO_COPYRIGHT="TRUE"
	else 
		echo "Print files with copyright statements"
	fi
fi

rm $FILE_CHECKFORCOPYRIGHT

find * | grep -v .svn | grep -v build/ > $FILE_CHECKFORCOPYRIGHT

NUMBER_OF_FILES=`< $FILE_CHECKFORCOPYRIGHT wc -l`
	
echo "Check recursively in directory if there is a copyright statement in the first $N lines"

for i in $(seq 1 1 $NUMBER_OF_FILES)
do
	CHECKFILE=`< $FILE_CHECKFORCOPYRIGHT head -n $i | tail -n 1`
	if [ -d $CHECKFILE ]; then
		# Is directory, don't cat the file
		#echo "Enter directory: $CHECKFILE"
		continue
	fi
	COPYRIGHT=`head -n $N $CHECKFILE | grep -i copyright | grep -v "BMH Associates"`
	if [ -z "$COPYRIGHT" ]; then
		if [ -n "$PRINT_NO_COPYRIGHT" ]; then
			echo "File $CHECKFILE has no copyright information"
			continue
		fi 
	else 
		if [ -z "$PRINT_NO_COPYRIGHT" ]; then
		        echo "File $CHECKFILE contains copyright: $COPYRIGHT"
		fi 
        fi
done
