/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */
#include <srActuator/HingeUniKarl.h>

//TODO: Remove the macros
//TODO: Is it necessary to have an extra uniKarl hinge class or is the hinge base class sufficient enough?

#define LINK_CFM 				0.001					// for CFM and ERP, read the manual of the ODE:
#define LINK_ERP 				0.95					// http://www.ode.org/ode-latest-userguide.html
#define FUDGE					0.1						//0.9
#define BOUNCE					0.0

#define PI						3.1415

// Torques are in N*(5cm).		1Nm ^= 20 N*(5cm)
#define MAX_TORQUE				2400				//600		//dInfinity
#define HOLDING_TORQUE			2400				//160
#define BRAKE_TORQUE			2400				//100
#define MAX_VELOCITY			15

namespace srCore {

HingeUniKarl::HingeUniKarl(dtCore::RefPtr<BodyBase> body1, dtCore::RefPtr<BodyBase> body2,
				float anchorX, float anchorY, float anchorZ,
				float axisX, float axisY, float axisZ, const std::string &hingeName) :
					HingeActuator(hingeName, "ukaHinge")
{

	this->init(body1, body2, anchorX, anchorY, anchorZ, axisX, axisY, axisZ);

	// set up hinge
	setODEParameter(dParamLoStop, -PI/2);				//PI/2
	setODEParameter(dParamHiStop, PI/2);
	setODEParameter(dParamStopERP, LINK_ERP);
	setODEParameter(dParamStopCFM, LINK_CFM);
	setODEParameter(dParamFudgeFactor, FUDGE);
	setODEParameter(dParamBounce, BOUNCE);
	setODEParameter(dParamFMax,MAX_TORQUE);

	this->setMaxTorque(MAX_TORQUE);
	this->setHoldingTorque(HOLDING_TORQUE);
	this->setBrakeTorque(BRAKE_TORQUE);
	this->setMaxVelocity(MAX_VELOCITY);
}

HingeUniKarl::~HingeUniKarl()
{

}
}
