/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */
#include <srActuator/WheelUniKarl.h>

// for CFM and ERP, read the manual of the ODE:
// http://www.ode.org/ode-latest-userguide.html
#define SUSPENSION_ERP 			0.9					//0.8
#define SUSPENSION_CFM 			0.3					//0.6
#define STOP_ERP 				0.9					//0.8
#define STOP_CFM 				0.3					//0.6

#define FUDGE					0.5
#define	BOUNCE					0.0

//#define PI						3.1415 // use M_PI if you need it

// Torques are in N*(5cm).		1Nm ^= 20N*(5cm)
#define MAX_TORQUE				1000

namespace srCore {
//! Constructor
/*! @param body1, body2 the bodies the hinge is attached to.
 * 	@param anchorX, anchorY, anchorZ the position of the hinge in the world coordinate system.
 *  @param axisX, axisY, axisZ the orientation of the axis of the hinge in the world coordinate system.
 *
 */
WheelUniKarl::WheelUniKarl(dtCore::RefPtr<BodyBase> body1, dtCore::RefPtr<BodyBase> body2,
					float anchorX, float anchorY, float anchorZ,
					float axis1_X, float axis1_Y, float axis1_Z,
					const std::string &name) :
						HingeActuator(name, "ukaWheel")

{

	this->init(	body1, body2,
				anchorX, anchorY, anchorZ,
				axis1_X, axis1_Y, axis1_Z);

	//set up wheel joint
//	setParam(dParamLoStop, -PI);
//	setParam(dParamHiStop, PI);
	//setParam(dParamSuspensionERP, SUSPENSION_ERP);
	//setParam(dParamSuspensionCFM, SUSPENSION_CFM);
	setODEParameter(dParamStopERP, STOP_ERP);
	setODEParameter(dParamStopCFM, STOP_CFM);
	setODEParameter(dParamFudgeFactor, FUDGE);
	setODEParameter(dParamBounce, BOUNCE);
	setODEParameter(dParamFMax, MAX_TORQUE);

	dJointSetHingeParam (joint, dParamLoStop, -4);
	dJointSetHingeParam (joint, dParamHiStop, 4);

}

WheelUniKarl::~WheelUniKarl()
{
}

}
