/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */
#include <srActuator/multiActuator.h>

namespace srCore {


MultiActuator::MultiActuator(dtCore::RefPtr<ActuatorBase> firstElement, const std::string name) :
	ActuatorBase(name, "MultiActuator")

{
	myActuators.push_back(firstElement);

	// add parameters of the first element to our parameter list
	myParameters = firstElement->getParameterList();


}

MultiActuator::~MultiActuator() {
	// TODO Auto-generated destructor stub
}



//! add actuator to container
void MultiActuator::addActuator(dtCore::RefPtr<ActuatorBase> actuator) {

	myActuators.push_back(actuator);

};

void MultiActuator::registerActuator() {

	for (unsigned int i=0; i < getNumActuators(); i++) {
		ActuatorBase::jointActuatorMap.insert(std::pair<dJointID, ActuatorBase*>(this->getHinge(i), this));

	}

}

//! Sets ode parameters for the actuators in the list. For more Information, see www.ode.org/ode-latest-user-guide.org.
inline void MultiActuator::setODEParameter(int parameter, float value) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->setODEParameter(parameter, value);

};

float MultiActuator::getODEParameter(int parameter) {

	return myActuators[0]->getODEParameter(parameter);


};


void MultiActuator::setParameter(const std::string &name, ParameterReturnValue  parameter) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->setParameter(name, parameter);

}

void MultiActuator::getParameter(const std::string &name, ParameterReturnValue  parameter) {

	myActuators[0]->getParameter(name, parameter);

};

//returns the jointID of the hinge.
inline dJointID MultiActuator::getJoint() {

	printf("Warning: This is a meta actuator. To get a hinge, use the getActuators function. Returning hinge of the first actuator \n");
	return myActuators[0]->getJoint();

};

unsigned int MultiActuator::getNumActuators() {

	return myActuators.size();

}

dJointID MultiActuator::getHinge(unsigned int index) {

	printf("Warning: This is a meta actuator. To get a hinge, use the getActuators function. Returning hinge of the first actuator \n");
	return ((index < myActuators.size()) ? myActuators[index]->getJoint() : NULL);

};
//! Setting a velocity.
/*! Motor tries to reach this velocity as fast as possible, but is constrained
 * by maxTorque
 */
void MultiActuator::setVel(float v, float deltaTime) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->setVel(v, deltaTime);

}

//! Setting a torque directly to a joint, can't be greater than maxTorque.
void MultiActuator::setTorque(float torque, float deltaTime) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->setTorque(torque, deltaTime);

}

//! Returns the current velocity of the joint.
float MultiActuator::getVel(float deltaTime) {return myActuators[0]->getVel(deltaTime);};

//! Returns the current position of the joint.
float MultiActuator::getPos() {return myActuators[0]->getPos();};

//! activate/deactivate the actuator
void MultiActuator::setActive(bool activate) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->setActive(active);

};

//! will be called in the TickLocal function of the robotActorBase class
void MultiActuator::update(float deltaSimTime) {

	for (unsigned int i=0; i<myActuators.size(); i++)
		myActuators[i]->update(deltaSimTime);

};


}
