/*
 * ScrewDrive.cpp
 *
 *  Created on: Nov 18, 2008
 *      Author: winkler
 */

#include <srActuator/screwDrive2.h>
#include <srCore/actuator/hingeActuator.h>
#include <stdexcept>
#include <cmath>

#include <dtUtil/log.h>

#define MAX(X,Y) (X>Y?X:Y)
#define MIN(X,Y) (X<Y?X:Y)

namespace srCore {


ScrewDrive2::ScrewDrive2(const std::string name):
	ActuatorBase(name, "ScrewDrive")
{
	
	d_m = 0.31;					//15.5mm
	my_Screw = 0.1;
	l = 4.95;					//9.9mm

	//speed is here the torque *gulp*
	F_div_M = d_m/2.0 * ((l + M_PI*my_Screw*d_m) / (M_PI*d_m - my_Screw*l));

}

ScrewDrive2::~ScrewDrive2() {
	// TODO Auto-generated destructor stub

}

void ScrewDrive2::registerActuator() {

	//ActuatorBase::jointActuatorMap.insert(std::pair<dJointID, ActuatorBase*>(frontActuator->getJoint(), frontActuator));
	//ActuatorBase::jointActuatorMap.insert(std::pair<dJointID, ActuatorBase*>(backActuator->getJoint(), backActuator));

}



void ScrewDrive2::setVel(float v, float deltaTime)
{
	velocity = v;

	float speedLeft, speedRight;

	if (secondScrewDrive != NULL) {
		if (this->isOnRightHandSide) {
			speedRight = v;
			speedLeft = secondScrewDrive->getVel(deltaTime);
		}
		else {
			speedLeft = v;
			speedRight = secondScrewDrive->getVel(deltaTime);
		}

		std::ostringstream msg; msg.clear(); msg.str("");
		msg << "Speed [left,right] = [" << speedLeft << "," << speedRight << "]";
		LOG_DEBUG(msg.str());

		float speedTrans = std::min(fabs(speedLeft), fabs(speedRight));
		float speedRot = fabs(speedLeft - speedRight);
		float sum = speedTrans + speedRot; // removed spurious calls to abs() below: both trans & rot speeds guaranteed positive now
		if (sum != 0) {
			speedTrans = speedTrans / sum;
			speedRot = speedRot / sum;
		}

		//std::cout << "v right =" << speedRight << ", v left = " << speedLeft <<std::endl;
		// actual super position of the movement
		if (this->isOnRightHandSide) {

			speedRight *= 0.01;
			speedLeft *= 0.01;

// 			dJointID f0 = getJoint(0);
// 			dJointID f1 = getJoint(1);
// 			dJointID b0 = secondScrewDrive->getJoint(0);
// 			dJointID b1 = secondScrewDrive->getJoint(1);

			BodyBase* leftBody = getAttachedBody();
			BodyBase* rightBody = secondScrewDrive->getAttachedBody();

// 			dVector3 pos0;
// 			dVector3 pos1;
// 			dBodyVectorToWorld(leftBody, 0,0,0, pos0);

// 			dVector3 pos0;
// 			dJointGetHingeAnchor (f0, pos0);
// 			dVector3 pos1;
// 			dJointGetHingeAnchor (f1, pos1);
// 
// 			dVector3 pos2;
// 			dJointGetHingeAnchor (b0, pos2);
// 			dVector3 pos3;
// 			dJointGetHingeAnchor (b1, pos3);

			
			float F_Screw_left = speedLeft * F_div_M;
			float F_Screw_right = -speedRight * F_div_M;

			float F_Roll_left = 2 * speedLeft / d_m;
			float F_Roll_right = 2 * speedRight / d_m;
			
			
			if (!leftBody->hasContact()) {
			  
			  F_Roll_left = 0;
			  F_Screw_left = 0;
			  
			}
			
			if (!rightBody->hasContact()) {
			  
			  F_Roll_right = 0;
			  F_Screw_right = 0;
			  
			}
			    

			float F_Screw = 0;
			if (F_Screw_left * F_Screw_right > 0) F_Screw = MIN(F_Screw_left, F_Screw_right);

			float F_Roll = 0;
			if (F_Roll_left * F_Roll_right > 0) F_Roll = MIN(F_Roll_left, F_Roll_right);

			float F_Rot_left = 0;
			float F_Rot_right = 0;
			if (F_Screw == 0) {
      
			   if ((leftBody->hasContact())	&& (rightBody->hasContact())) {
			    
			      if (fabs(F_Roll_left) > fabs(F_Roll_right)) F_Rot_left = F_Screw_left + F_Screw_right;
			      else F_Rot_right = F_Screw_right + F_Screw_left;
			    
			   }
			  
			 
			  
			}
			
			if (F_Roll == 0) {
			  
			   if ((leftBody->hasContact())	&& (rightBody->hasContact())) {
			    
			      if (fabs(F_Screw_left) > fabs(F_Screw_right)) F_Rot_left = F_Screw_left - F_Screw_right;
			      else F_Rot_right = F_Screw_right - F_Screw_left;
			      
			   }
			  
			}

// std::cout << "F_Roll = " << F_Roll << ", F_Screw = " << F_Screw << std::endl;

// std::cout << osg::Vec3(pos0[0], pos0[1],pos0[2]) << std::endl;

			
			  dBodyAddRelForceAtRelPos    (leftBody->GetBodyID(), 0, F_Roll, F_Screw + F_Rot_left, -1, 1, 0);	
			  dBodyAddRelForceAtRelPos    (rightBody->GetBodyID(), 0, F_Roll, F_Screw + F_Rot_right, -1, -1, 0);
			//dBodyAddRelForceAtPos    (backBody, 0, F_Roll, -F_Screw*100, (pos2[0] + pos3[0]) / 2, (pos2[1] + pos3[1]) / 2, (pos2[2] + pos3[2]) / 2);
			//dBodyAddRelForce    (backBody, 0, 0, -F_Roll);//, F_Screw*100);


//			frontActuator->setODEParameter(dParamVel, speedRight);
//			backActuator->setODEParameter(dParamVel, -speedRight * speedTrans + speedRot * speedLeft);
//			secondScrewDrive->frontActuator->setODEParameter(dParamVel,  speedLeft);
//			secondScrewDrive->backActuator->setODEParameter(dParamVel, -speedLeft * speedTrans + speedRot * speedRight);

		} else {
  
			secondScrewDrive->setVel(secondScrewDrive->getVel(deltaTime), deltaTime);
//			frontActuator->setODEParameter(dParamVel,  speedLeft);
//			backActuator->setODEParameter(dParamVel, -speedLeft * speedTrans + speedRot * speedRight);
//			secondScrewDrive->frontActuator->setODEParameter(dParamVel,  speedRight);
//			secondScrewDrive->backActuator->setODEParameter(dParamVel, -speedRight * speedTrans + speedRot * speedLeft);
		}
	}

#ifdef	AUDIO_ON
	if(screwSound != NULL)
	{
		float speed = fabs(v);
		if(speed > 0.0)
		{
			screwSound->SetGain(0.1f+speed/10);
			screwSound->SetPitch(0.5f+speed/100);
			if (!screwSound->IsPlaying())
				screwSound->Play();
		}
		else if(screwSound->IsPlaying())
			screwSound->Stop();
	}
#endif
}

}
