//! Basic hinge class.

/*!
 *
 * This class simulates a hinge joint. Can be used for all rotatorial joints with just one axis.
 * For example, for hinges or wheels, what are fixed.
 *
 * @author Lutz Winkler
 *
 */

//TODO: Include AMotor in such a way the joint can reach a given velocity using a torque not bigger than the maximum torque
//TODO: Remove the hack in the gotoAngle function
//TODO: What control mechanism will be used for the 3D drive later? (POSITION, VELOCITY, FORCE)
//TODO: clean up code


#include <math.h>

#include <srCore/actuator/hingeActuator.h>

namespace srCore {

/**
 * All parameters are pushed on the local myParameters vector.
 */
HingeActuator::HingeActuator(const std::string name,  const std::string type) :
	ActuatorBase(name, type),
	joint(), motor(),
	distToBody1(0), distToBody2(0),
	maxTorque(0),
	maxHoldingTorque(0),
	maxBrakeTorque(0),
	maxVelocity(0),
	isAtConstraint(false),
	paramAngle(0), paramVelocity(0), paramFMax(0),
	paramDesiredVelocity(0), paramLoStop(0), paramHiStop(0), paramDesiredAngle(0),
	zeroAngle (0)
{

	ObjectParameter sp;

	sp.valueName = "Angle";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	sp.valueName = "AngleVelocity";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	sp.valueName = "FMax";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	sp.valueName = "DesiredVelocity";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	sp.valueName = "LoStop";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	sp.valueName = "HiStop";
	sp.valueType = typeid(float*).name();
	myParameters.push_back(sp);

	paramHiStop = 1.57;
	paramLoStop = -1.57;
}


HingeActuator::~HingeActuator()
{

	dJointDestroy(this->joint);
	dJointDestroy(this->motor);
}

//! Initializes the hinge. Needs to be called before using the hinge.
void HingeActuator::init(dtCore::RefPtr<BodyBase> myBody1, dtCore::RefPtr<BodyBase> myBody2,
					float anchorX, float anchorY, float anchorZ,
					float axisX, float axisY, float axisZ)
{

	paramDesiredAngle = 0.0;

	dWorldID worldID = myBody1->GetSceneParent()->GetWorldID();
#ifdef VERBOSE
	printf("init robot\n");
#endif
	dBodyID body1 = myBody1->GetBodyID();
	dBodyID body2 = myBody2->GetBodyID();

	myBody = myBody1;

	// create Hinge
	this->joint = dJointCreateHinge(worldID, 0);
	this->motor = dJointCreateAMotor (worldID, 0);


	// attach joint and motor
	// TODO: Do we need the ode motor, what is the motor doing exactly?
	dJointAttach(this->joint, body1, body2);
//	dJointAttach(this->motor, body1, body2);


	//set relation of the parts of the replicator
	dJointSetHingeAnchor(this->joint, anchorX, anchorY, anchorZ);

	// set joint properties
	dJointSetHingeAxis(this->joint, axisX, axisY, axisZ);

	// set motor properties
//	dJointSetAMotorNumAxes(this->motor,1);
//	dJointSetAMotorAxis(this->motor, 0, 1, axisX, axisY, axisZ);
//	dJointSetAMotorMode(this->motor, dAMotorUser);
//
//	dJointSetAMotorParam(this->motor, dParamVel, 0);
//	dJointSetAMotorParam(this->motor, dParamFMax, dInfinity);

	dJointSetHingeParam (joint, dParamLoStop, paramLoStop);
	dJointSetHingeParam (joint, dParamHiStop, paramHiStop);

	// pre-initializing, if maxTorques are zero, they will be ignored
	this->maxTorque = 0;
	this->maxHoldingTorque = 0;
	this->maxBrakeTorque = 0;
	this->maxVelocity = 1;

}

// type check and return corresponding parameter value
void HingeActuator::getParameter(const std::string &name, ParameterReturnValue  parameter) {

	if (name == "Angle") {

		// check parameter type
		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");

		// cast void* to float*
		float* data = reinterpret_cast<float*> (parameter.pValue);

		// get parameter
		paramAngle = dJointGetHingeAngle (joint) + zeroAngle;

		// write to given pointer
		*data = paramAngle;
		parameter.pValue = data;
	}

	else if (name == "AngleVelocity") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramVelocity = dJointGetHingeAngleRate (joint);// / getGameManager()->getDeltaSimTime() //<<-- oder so
		*data = paramVelocity;
		parameter.pValue = data;
	}

	else if (name == "FMax") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramFMax = dJointGetHingeParam (joint, dParamFMax);
		*data = paramFMax;
		parameter.pValue = data;
	}

	else if (name == "DesiredVelocity") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramDesiredVelocity = dJointGetHingeParam (joint, dParamVel);
		*data = paramDesiredVelocity;
		parameter.pValue = data;
	}

	else if (name == "LoStop") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramLoStop = dJointGetHingeParam (joint, dParamLoStop);
		*data = paramLoStop;
		parameter.pValue = data;
	}

	else if (name == "HiStop") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramHiStop = dJointGetHingeParam (joint, dParamHiStop);
		*data = paramHiStop;
		parameter.pValue = data;
	}

	else ActuatorBase::getParameter(name, parameter);//printf("Warning: Parameter %s unknown \n", name.data());

}

// type check and set corresponding parameter
void HingeActuator::setParameter(const std::string &name, ParameterReturnValue  parameter) {

	if (name == "Angle") {

		printf("Warning; Cannot set angle directly. Use FMax and DesiredVelocity instead. \n");

	}

	else if (name == "AngleVelocity") {

		printf("Warning: Cannot set angle velocity directly. Use FMax and DesiredVelocity instead. \n");

	}

	else if (name == "FMax") {

		// check parameter type
		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");

		// cast void* to float*
		float* data = reinterpret_cast<float*> (parameter.pValue);

		// set parameter
		paramFMax = *data;
		dJointSetHingeParam (joint, dParamFMax, paramFMax);

	}

	else if (name == "DesiredVelocity") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramDesiredVelocity = *data;
		dJointSetHingeParam (joint, dParamVel, paramDesiredVelocity);

	}

	else if (name == "LoStop") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramLoStop = *data;
		dJointSetHingeParam (joint, dParamLoStop, paramLoStop);

	}

	else if (name == "HiStop") {

		if (parameter.type != typeid(float*).name()) printf("Parameter type needs to be float*. Casting could lead to errors.  \n");
		float* data = reinterpret_cast<float*> (parameter.pValue);
		paramHiStop = *data;
		dJointSetHingeParam (joint, dParamHiStop, paramHiStop);

	}

	else ActuatorBase::setParameter(name, parameter);//printf("Warning: Parameter %s unknown \n", name.data());

}

//! Returns the ODE hinge joint.
dJointID HingeActuator::getJoint() {

	return this->joint;

}

//! Register Actuator to the simulation
void HingeActuator::registerActuator()
{
	ActuatorBase::jointActuatorMap.insert(std::pair<dJointID, ActuatorBase*>(this->getJoint(), this));
}

//! When the initial joint position is unequal to zero, the initial joint angle needs to be given as the zeroAngle.
void HingeActuator::setZeroAngle(float angle) {

	zeroAngle = angle;

	paramLoStop = this->getODEParameter(dParamLoStop);
	paramHiStop = this->getODEParameter(dParamHiStop);

	paramHiStop -= angle;
	paramLoStop -= angle;

	this->setODEParameter(dParamLoStop, paramLoStop);
	this->setODEParameter(dParamHiStop, paramHiStop);

	gotoAngle(angle);

}

//! Position control
//TODO: add parameter for min/max angle value (currently fixed (90 degrees)),
void HingeActuator::gotoAngle(float angle) {

	float F;
//	double epsilon = 1e-5;
//	double e = 0.001;
	F = 2800;// * (fabs(getPos() - angle));


//	if (fabs(angle - getPos()) <= e) return;

	angle -= zeroAngle;

	paramLoStop = this->getODEParameter(dParamLoStop);
	paramHiStop = this->getODEParameter(dParamHiStop);

	if (angle > paramHiStop) angle = paramHiStop;
	if (angle < paramLoStop) angle = paramLoStop;

	paramDesiredAngle = angle;


// ----------------------------------------------------------------------------------------------
//old stuff that constrians the hinge to its current position +/- epsilon:

//	dJointSetAMotorParam(this->motor, dParamVel, 0);
//	dJointSetAMotorParam(this->motor, dParamFMax, dInfinity);
//
//	if (getPos() < paramLoStop) angle = paramLoStop;//getPos() + 0.01;
//	if (getPos() > paramHiStop) angle = paramHiStop;//getPos() - 0.01;
//
//
//	if (getPos() > angle) {
//
//		v = -1;// 0.000001 * (angle - getPos());
//
////		dJointSetHingeParam (joint, dParamLoStop, (angle<paramLoStop ? paramLoStop : angle));
////		dJointSetHingeParam (joint, dParamHiStop, (getPos()>paramHiStop ? paramHiStop : getPos()));
//
//		this->setODEParameter(dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, 3800);
//		this->setODEParameter(dParamFMax, F);
//		return;
//	}
//
//	if (getPos() < angle) {
//
//		v = 1;//;0.000001 * (angle - getPos());
//
////		dJointSetHingeParam (joint, dParamLoStop, (getPos()<paramLoStop ? paramLoStop : getPos()));
////		dJointSetHingeParam (joint, dParamHiStop, (angle>paramHiStop ? paramHiStop : angle));
//
//		this->setODEParameter(dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, 3800);
//		this->setODEParameter(dParamFMax, F);
//		return;
//	}

	//if (((angle > getPos()) && (getPos() > -1.57)) || (getPos() < paramLoStop)) {

	//if (fabs(angle - getPos()) > 0.1)
//	if (angle > getPos()) {
//
//
//		v = 1e-16;// maxSpeed / 1000000;
//		v = -10;
//		//v = -0.01 * (-getPos() + angle);
//
//		float min = (getPos() < angle) ? getPos() : angle;
//		float max = (getPos() > angle) ? getPos() : angle;
//
//		min -= epsilon;
//		max += epsilon;
//
//		min = (min < paramLoStop) ? paramLoStop : min;
//		max = (max > paramHiStop) ? paramHiStop : max;
//
//		dJointSetHingeParam (joint, dParamLoStop, min);
//		dJointSetHingeParam (joint, dParamHiStop, max);
//
//		this->setODEParameter(dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, 3800);
//		this->setODEParameter(dParamFMax, F);							//3800 = 9.5 Nm
//		//		this->setODEParameter(dParamFMax, 0);
//
//
//	}
//	else {
//
//	//	std::cout << "conn0: angle < getPos" << std::endl;
//
////	if (((angle > getPos()) && (getPos() > -1.57)) || (getPos() > 1.57)) {
//		v = -1e-16;// -maxSpeed / 1000000;
//		v= 10;
//
//		//v = -0.01 * (-getPos() + angle);
//		float min = (getPos() < angle) ? getPos() : angle;
//		float max = (getPos() > angle) ? getPos() : angle;
//
//		min -= epsilon;
//		max += epsilon;
//
//		min = (min < paramLoStop) ? paramLoStop : min;
//		max = (max > paramHiStop) ? paramHiStop : max;
//
//		dJointSetHingeParam (joint, dParamHiStop, max);
//		dJointSetHingeParam (joint, dParamLoStop,  min);
//
//
//		this->setODEParameter(dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, v);
//		//		dJointSetAMotorParam(this->motor, dParamVel, 3800);
//		this->setODEParameter(dParamFMax, F);
//		//		this->setODEParameter(dParamFMax, 0);
//
//	}



}

//! Sets the parameters of the hinge as been described on the ode webpage.
void HingeActuator::setODEParameter(int parameter, float value) {

	dJointSetHingeParam(this->joint, parameter, value);

}

//! Sets the parameters of the hinge as been described on the ode webpage.
float HingeActuator::getODEParameter(int parameter) {

	return dJointGetHingeParam(this->joint, parameter);

}

void HingeActuator::setMaxTorque(float torque)
{
	if (torque >= 0)
		this->maxTorque = torque;

}

void HingeActuator::setHoldingTorque(float torque)
{
	if (torque >= 0)
		this->maxHoldingTorque = torque;

}

void HingeActuator::setBrakeTorque(float torque)
{
	if (torque >= 0)
		this->maxBrakeTorque = torque;

}

void HingeActuator::setMaxVelocity(float v_max)
{
	this->maxVelocity = v_max;
}

//TODO: Use AMotor instead of the calculateTorqueToStop function
//! DISABLED Estimates the torque value that is needed to stop the hinge in the next time step (deltaTime).
double HingeActuator::calculateTorqueToStop(float deltaTime) {

//
//
//	 //dxJointHinge* joint = (ode::dxJointHinge*)j;
//	 double torque;
//	 double angularVelocity;
//	 double angularAcceleration;
//
//	 dBodyID body1 = dJointGetBody (this->joint, 0);
//	 dBodyID body2 = dJointGetBody (this->joint, 1);
//	 dMass mass1;
//	 dMass mass2;
//
//	 dBodyGetMass(body1, &mass1);
//	 dBodyGetMass(body2, &mass2);
//
//	 // calculating the angular velocity from the AngleRate and the delta time.
//	 angularVelocity = dJointGetHingeAngleRate(this->joint);
//	 angularAcceleration = -angularVelocity / deltaTime;
//
//	 // estimates the torque needed to stop the hinge.
//	 torque = 0.5 * (mass1.mass * this->distToBody1 + mass2.mass * this->distToBody2) * angularAcceleration;
//	 //torque = 0.5 * (this->distToBody1 + this->distToBody2) * angularAcceleration;

	 return 0;//torque;


}

//! DISABLED: setting torque directly. if torque equals zero, motor is off.
void HingeActuator::setTorque(float torque, float deltaTime)
{
//
//	float angleRate = dJointGetHingeAngleRate(this->joint);
//
//	// abs(torque) has to bee equal or less than maxTorque
//	if (this->maxTorque != 0) {
//
//		if (torque > this->maxTorque) torque = this->maxTorque;
//		if (torque < -this->maxTorque) torque = -this->maxTorque;
//
//	}
//
//	//hinge can't exceed its constraints.
//	if (dJointGetHingeAngle(this->joint) > dJointGetHingeParam(this->joint, dParamHiStop)) {
//
//		torque = this->calculateTorqueToStop(deltaTime);
//		dJointSetHingeParam (this->joint, dParamFMax, 0);
//		dJointAddHingeTorque(this->joint, torque);
//		return;
//
//	}
//
//	else if (dJointGetHingeAngle(this->joint) < dJointGetHingeParam(this->joint, dParamLoStop)) {
//
//		torque = this->calculateTorqueToStop(deltaTime);
//		dJointSetHingeParam (this->joint, dParamFMax, 0);
//		dJointAddHingeTorque(this->joint, torque);
//		return;
//
//	}
//
//	//motor off - hinge is going to stop, braking/holding torques can not exceed the corresponding max torques.
//	if (torque == 0) {
//
//		this->setODEParameter(dParamVel, 0);
//		if (angleRate == 0) this->setODEParameter(dParamFMax, this->maxHoldingTorque);
//		else				this->setODEParameter(dParamFMax, this->maxBrakeTorque);
//
//	}
//
//
//	else {
//
//
//
//		//motor on - set torque, but dont exceed max velocity:
//		this->setODEParameter(dParamVel, this->maxVelocity);
//		this->setODEParameter(dParamFMax, torque);
//		//dJointAddHingeTorque(this->joint, torque);
//
//	}

}

//TODO: clean up the setVel function
//! DISABLED: setting the velocity. Maximum torques can't be exceeded to reach the velocity.
// disabled
void HingeActuator::setVel(float v, float deltaTime)
{
//	//float torque;
///*
//	if (this->isAtConstraint) {
//
//		this->setODEParameter(dParamFMax, 2000);
//		this->setODEParameter(dParamVel, -v);
//		this->isAtConstraint = false;
//
//		return;
//
//	}
//
//	else*/
//
//	// constraints have been reached.
//	if ((v>=0) && (dJointGetHingeAngle(this->joint) > dJointGetHingeParam(this->joint, dParamHiStop))) {
//
//		//printf("hi\n");
//		/*torque = this->calculateTorqueToStop(deltaTime);
//		this->setODEParameter(dParamFMax, 3000);
//		this->setODEParameter(dParamVel, -v);
//		this->isAtConstraint = true;
//		//dJointAddHingeTorque(this->joint, -torque);*/
//		this->setODEParameter(dParamVel, 0);
//		this->setODEParameter(dParamFMax, 4800);						//300
//		return;
//
//	}
//
//	else if ((v<=0) && (dJointGetHingeAngle(this->joint) < dJointGetHingeParam(this->joint, dParamLoStop))) {
//
//		//printf("lo\n");
//		/*torque = this->calculateTorqueToStop(deltaTime);
//		this->setODEParameter(dParamFMax, 3000);
//		this->setODEParameter(dParamVel, -v);
//		this->isAtConstraint = true;*/
//		//dJointAddHingeTorque(this->joint, -torque);
//		this->setODEParameter(dParamVel, 0);
//		this->setODEParameter(dParamFMax, 4800);
//		return;
//
//	}
//
//	// get to the desired velocity, using maximum torque if necessary.
//	else {
//
//		/*if ((dJointGetHingeAngle(this->joint) > dJointGetHingeParam(this->joint, dParamHiStop)) ||
//			(dJointGetHingeAngle(this->joint) < dJointGetHingeParam(this->joint, dParamLoStop)))
//
//				this->setODEParameter(dParamVel, 0);
//		else*/
//
//		if (v >=  maxVelocity) {
//			v =  maxVelocity;
//			//printf("max speed \n");
//		}
//		if (v <= -maxVelocity) {
//			v = -maxVelocity;
//			//printf("max speed \n");
//		}
//
//
//		//printf("FMax = %f ", dJointGetAMotorParam(this->motor,dParamFMax) );
//
//		//this->setODEParameter(dParamVel, v);
//
//		dJointSetAMotorParam(this->motor,dParamVel, v);
//		dJointSetAMotorParam(this->motor,dParamFMax, dInfinity);
//
//		//printf(" %f\n", this->maxTorque);
//		//this->setODEParameter(dParamFMax, 10000);// this->maxTorque);
//
//	}

}

// deltaTime needs to be provided by the tick message:
//
// const dtGame::TickMessage &tick = static_cast<const dtGame::TickMessage&>(tickMessage);
// float deltaSimTime = tick.GetDeltaSimTime();

// TODO: obtain the delta time otherwise?
float HingeActuator::getVel(float deltaTime)
{
	dReal angleRate = dJointGetHingeAngleRate(this->joint);
	return (angleRate / deltaTime);

}

float HingeActuator::getPos()
{
	return (dJointGetHingeAngle(this->joint) - zeroAngle);
}

float HingeActuator::getAngleRate()
{
	return (dJointGetHingeAngleRate(this->joint));
}


void HingeActuator::update(float deltaSimTme) {

	float epsilon = 0.02;
	float v = maxVelocity;
	float F = maxTorque;

	paramLoStop = getODEParameter(dParamLoStop);
	paramHiStop = getODEParameter(dParamHiStop);

	if (paramDesiredAngle > paramHiStop - epsilon) paramDesiredAngle = paramHiStop - epsilon;
	if (paramDesiredAngle < paramLoStop + epsilon) paramDesiredAngle = paramLoStop + epsilon;

	float internalDesiredAngle = paramDesiredAngle - zeroAngle;

	float delta = getAngleRate();

	if (delta / (internalDesiredAngle - getPos()) > 0.2)
	//if (fabs(internalDesiredAngle - getPos()) < fabs(delta))
		{

		v *= (internalDesiredAngle - getPos()) / delta;
		F *= (internalDesiredAngle - getPos()) / delta;

	}

	if (fabs(internalDesiredAngle - getPos()) < epsilon) {
		this->setODEParameter(dParamFMax, 10000);
		this->setODEParameter(dParamVel, 0);
		return;
	}

	if (getPos() > internalDesiredAngle) {
		this->setODEParameter(dParamFMax, F);
		this->setODEParameter(dParamVel, -v);
		return;
	}
	if (getPos() < internalDesiredAngle) {
		this->setODEParameter(dParamFMax, F);
		this->setODEParameter(dParamVel, v);
		return;
	}




}

}
