/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */

#include <srCore/actuator/wheelActuator.h>

namespace srCore {


WheelActuator::WheelActuator(const std::string name,  const std::string type) :
	ActuatorBase(name, type)
{
}

WheelActuator::~WheelActuator()
{

	dJointDestroy(this->joint);
}


//! initializes the connector. Needs to be called before using the wheel.
void WheelActuator::init(dtCore::RefPtr<BodyBase> myBody1, dtCore::RefPtr<BodyBase> myBody2,
						float anchorX, float anchorY, float anchorZ,
						float axis1_X, float axis1_Y, float axis1_Z,
						float axis2_X, float axis2_Y, float axis2_Z)

{
	dWorldID worldID = myBody1->GetSceneParent()->GetWorldID();

	//create Wheel using the Hinge2 joint
	this->joint = dJointCreateHinge2(worldID, 0);
	dJointAttach(this->joint, myBody1->GetBodyID(), myBody2->GetBodyID());
	myBody = myBody1;

	//set relation of the parts of the replicator
	dJointSetHinge2Anchor(this->joint, anchorX, anchorY, anchorZ);

	//set joint axis
	dJointSetHinge2Axis1(this->joint, axis1_X, axis1_Y, axis1_Z);
	dJointSetHinge2Axis2(this->joint, axis2_X, axis2_Y, axis2_Z);
}

void WheelActuator::registerActuator()
{

		ActuatorBase::jointActuatorMap.insert(std::pair<dJointID, ActuatorBase*>(this->getJoint(), this));

};

dJointID WheelActuator::getWheel() {

	return this->joint;

}

//!  Sets ode parameters for the joint. For more Information, see www.ode.org/ode-latest-user-guide.org.
void WheelActuator::setODEParameter(int parameter, float value) {

	dJointSetHinge2Param(this->joint, parameter, value);

}

}
