/* -*-c++-*-
 * Delta3D Open Source Game and Simulation Engine
 * Copyright (C) 2005, BMH Associates, Inc.cout
 *
 * Input Component
 *
 * @author Lutz Winkler
 * @author Adrian Friebel
 */

#include <srCore/inputComponent.h>
#include <srCore/arenaComponent.h>
#include <srCore/cameraComponent.h>
#include <srCore/simulationEvents.h>

#include <iostream>

#include <dtUtil/log.h>

//! Defines the number of game ticks before a next screenshot is taken
#define NO_OF_TICKS_UNTIL_SCREENSHOT 0

using namespace srCore;

/**
 * The SimulationInputComponent contains all functionality that captures mouse and keyboard
 * events. Use the declaration order of takeScreenshot etc in simulationInputComponent.h.
 * @param name
 * @param robCtrlEna
 * @return
 */
SimulationInputComponent::SimulationInputComponent(const std::string &name, bool robCtrlEna) : dtGame::BaseInputComponent(name),
		takeScreenshot (false),
		rotAroundRobot (false),
		robotControlEnabled(robCtrlEna),
		ticks(0){

	LOG_INFO("Creation and initialisation");

	//	dtGame::GameApplication *GA = dtGame::GameApplication::GetInstance(0);
	//	mScreenShotTaker = new ScreenShotCallback(RGB_CAMERA, GA->GetCamera()->GetOSGCamera()->getViewport());

	// Create a few game events and register them with the event manager.

	mMoveActorUp = new dtDAL::GameEvent( MOVE_ACTUATOR1_FWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mMoveActorUp);

	mMoveActorDown = new dtDAL::GameEvent( MOVE_ACTUATOR1_BWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mMoveActorDown);

	mMoveActorLeft = new dtDAL::GameEvent( MOVE_ACTUATOR2_FWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mMoveActorLeft);

	mMoveActorRight = new dtDAL::GameEvent( MOVE_ACTUATOR2_BWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mMoveActorRight);

	mAccelerate = new dtDAL::GameEvent( MOVE_ACTUATOR3_FWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mAccelerate);

	mDecelerate = new dtDAL::GameEvent( MOVE_ACTUATOR3_BWD );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mDecelerate);

	mSavingPicture = new dtDAL::GameEvent( CAMERA1_SAVE_IMAGE );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mSavingPicture);

	mSaveScreenShot = new dtDAL::GameEvent( TAKE_SCREEN_SHOT_TOGGLE );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mSaveScreenShot);


	mZoomIn = new dtDAL::GameEvent( CAMERA1_ZOOM_IN );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mZoomIn);

	mZoomOut = new dtDAL::GameEvent( CAMERA1_ZOOM_OUT );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mZoomOut);

	mConnectAll = new dtDAL::GameEvent( CONNECT_ALL );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mConnectAll);

	mDisconnectAll = new dtDAL::GameEvent( DISCONNECT_ALL );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mDisconnectAll);

	mCamMoveUp = new dtDAL::GameEvent( CAMERA1_MOVE_UP );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mCamMoveUp);

	mCamMoveDown = new dtDAL::GameEvent( CAMERA1_MOVE_DOWN );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mCamMoveDown);

	mCamMoveLeft = new dtDAL::GameEvent( CAMERA1_MOVE_LEFT );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mCamMoveLeft);

	mCamMoveRight = new dtDAL::GameEvent( CAMERA1_MOVE_RIGHT );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mCamMoveRight);


	mTimeScaleChange = new dtDAL::GameEvent( TIME_SCALE_CHANGED );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mTimeScaleChange);

	mSelectRobot = new dtDAL::GameEvent( CHANGE_ROBOT_SELECTION );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mSelectRobot);

	mMouseLBClick = new dtDAL::GameEvent( MOUSE_LEFT_CLICK );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mMouseLBClick);

	mSwitchControl = new dtDAL::GameEvent( SWITCH_CONTROL );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mSwitchControl);

#ifdef PARTNER_ALMENDE
mToggleLearnToDrive = new dtDAL::GameEvent( TOGGLE_LEARN_TO_DRIVE );
dtDAL::GameEventManager::GetInstance().AddEvent(*mToggleLearnToDrive);

mToggleLearnBinLoc = new dtDAL::GameEvent( TOGGLE_LEARN_BIN_LOC );
dtDAL::GameEventManager::GetInstance().AddEvent(*mToggleLearnBinLoc);

mToggleImageSaliency = new dtDAL::GameEvent( TOGGLE_IMAGE_SAL );
dtDAL::GameEventManager::GetInstance().AddEvent(*mToggleImageSaliency);

mToggleAudioSaliency = new dtDAL::GameEvent( TOGGLE_AUDIO_SAL );
dtDAL::GameEventManager::GetInstance().AddEvent(*mToggleAudioSaliency);
#endif


mPauseSim = new dtDAL::GameEvent( PAUSE_SIM );
dtDAL::GameEventManager::GetInstance().AddEvent(*mPauseSim);

mResetSim = new dtDAL::GameEvent( RESET_SIM );
dtDAL::GameEventManager::GetInstance().AddEvent(*mResetSim);

mDestroyActor = new dtDAL::GameEvent( DESTROY_ACTOR );
dtDAL::GameEventManager::GetInstance().AddEvent(*mDestroyActor);

mDestroyAllActor = new dtDAL::GameEvent( DESTROY_ALL_ACTOR );
dtDAL::GameEventManager::GetInstance().AddEvent(*mDestroyAllActor);

mCreateActor = new dtDAL::GameEvent( CREATE_ACTOR );
dtDAL::GameEventManager::GetInstance().AddEvent(*mCreateActor);

PrintPossibilities();
}

SimulationInputComponent::~SimulationInputComponent()
{

}

void SimulationInputComponent::PrintPossibilities() {
	int keycount = 23;
	int keys[] = {
			'c', 'j', 'u', 'f', 'h',
			't', 'g', 'p', 'r', 'R', 'o',
			'M', 'm', 'n', 'b', 'v',
			'k', 'l', 'y', 'x', '1', '2', '3'};
	LOG_INFO("Defined input keys");
	for (int i = 0; i < keycount; i++) {
		switch(keys[i]) {
		case 'c': LOG_INFO("c: switch control"); break;
		case 'j': LOG_INFO("j: decelerate"); break;
		case 'u': LOG_INFO("u: accelerate"); break;
		case 'f': LOG_INFO("f: move actor to the left"); break;
		case 'h': LOG_INFO("h: move actor to the right"); break;
		case 't': LOG_INFO("t: move actor up"); break;
		case 'g': LOG_INFO("g: move actor down"); break;
		case 'p': LOG_INFO("p: toggle saving pictures for video"); break;
		case 'r': LOG_INFO("r: pause/unpause the simulation"); break;
		case 'R': LOG_INFO("r: reset the simulation"); break;
		case 'o': LOG_INFO("o: select robot"); break;
		case 'M': LOG_INFO("M: communicate message to all robots"); break;
		case 'm': LOG_INFO("m: move camera to the right"); break;
		case 'n': LOG_INFO("n: move camera to the left"); break;
		case 'b': LOG_INFO("b: move camera up"); break;
		case 'v': LOG_INFO("v: move camera down"); break;
		case 'k': LOG_INFO("k: connect all"); break;
		case 'l': LOG_INFO("l: disconnect all"); break;
		case 'y': LOG_INFO("y: speed up simulation time"); break;
		case 'x': LOG_INFO("x: slow down simulation time"); break;
		case '1': LOG_INFO("1: add new robots (as described in the scene file)"); break;
		case '2': LOG_INFO("2: delete selected robot"); break;
		case '3': LOG_INFO("3: delete all robots"); break;
		}
	}
}

////////////////////////////////////////////////////////////////////
bool SimulationInputComponent::HandleKeyPressed(const dtCore::Keyboard* keyboard, int key)
{
	bool handled = true;
	std::vector< dtGame::GameActorProxy* > allGameActors;
	std::ostringstream oss;

	switch(key) {

	case 'c': sendMessageToAll(*mSwitchControl); break;
	case 'j': sendMessageToRobot(*mDecelerate); break;
	case 'u': sendMessageToRobot(*mAccelerate); break;
	case 'f': sendMessageToRobot(*mMoveActorLeft); break;
	case 'h': sendMessageToRobot(*mMoveActorRight); break;
	case 't': sendMessageToRobot(*mMoveActorUp); break;
	case 'g': sendMessageToRobot(*mMoveActorDown); break;

	// toggle saving pictures for video
	case 'p':
		takeScreenshot ^= 1;
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_INFO, __func__, __LINE__,
				"Take screenshot is now set to %i", takeScreenshot);
		break;

	case 'o':
	{
		SimulationArenaComponent* sac = dynamic_cast<SimulationArenaComponent*>(GetGameManager()->GetComponentByName(SimulationArenaComponent::DEFAULT_NAME));
		if (sac) sac->selectNextRobot();
		//sendMessageToAll(*mSelectRobot);
		break;
	}

	case 'M':
	{
		SimulationArenaComponent *comp = static_cast<SimulationArenaComponent*>(GetGameManager()->GetComponentByName(SimulationArenaComponent::DEFAULT_NAME));
		int index = comp->getRobFocIndex();

		if (index < 0) { LOG_INFO("Index is smaller than 0"); break; }
		if (index > (int)allGameActors.size()) { LOG_INFO("Not so many game actors"); break; }

		dtCore::RefPtr<RobotActorBase> selectedActor = comp->getRobotsInScene()[index];

		commMessage();

		std::vector<RobotActorBase* > cr = selectedActor->getConnectedRobots();
		std::ostringstream oss;

		oss << "no of robots = " << cr.size();
		LOG_INFO(oss.str()); oss.clear();
		LOG_INFO("connected robots:");

		for (unsigned int i=0; i<cr.size(); i++)
			oss << " " << cr[i]->getID();
		LOG_INFO(oss.str());
		break;
	}
	case 'm': sendMessageToRobot(*mCamMoveRight); break;
	case 'n': sendMessageToRobot(*mCamMoveLeft); break;
	case 'b': sendMessageToRobot(*mCamMoveUp); break;
	case 'v': sendMessageToRobot(*mCamMoveDown); break;
	case 'k': sendMessageToRobot(*mConnectAll); break;
	case 'l': sendMessageToRobot(*mDisconnectAll); break;

#ifdef PARTNER_ALMENDE
	// Controller commands
	case 'I':
		sendMessageToRobot(*mToggleImageSaliency);
		break;
	case 'A':
		sendMessageToRobot(*mToggleAudioSaliency);
		break;
	case '4':
		sendMessageToRobot(*mToggleLearnToDrive);
		break;
	case '5':
		sendMessageToRobot(*mToggleLearnBinLoc);
		break;
#endif
		// '+' Speed up simulation time factor;

		//TODO: move function to change time scale to arena component
	case 'y':
		GetGameManager()->ChangeTimeSettings(GetGameManager()->GetSimulationTime(),
				GetGameManager()->GetTimeScale() * 1.1, GetGameManager()->GetSimulationClockTime());

		oss << GetGameManager()->GetTimeScale();
		LOG_INFO(oss.str());oss.clear();
		sendMessageToAll(*mTimeScaleChange);
		break;

		// '-' Slow down the simulation time factor;
	case 'x':
		GetGameManager()->ChangeTimeSettings(GetGameManager()->GetSimulationTime(),
				GetGameManager()->GetTimeScale() * 0.9, GetGameManager()->GetSimulationClockTime());

		oss << GetGameManager()->GetTimeScale();
		LOG_INFO(oss.str());oss.clear();
		sendMessageToAll(*mTimeScaleChange);
		break;

	case 'r':												//nur zum Test hier, soll wieder weg
		sendMessageToAll(*mPauseSim);
		break;

	case 'R':												//nur zum Test hier, soll wieder weg
		sendMessageToAll(*mResetSim);
		break;

	case '3':
		sendMessageToAll(*mDestroyAllActor);
		break;
	case '2':
		sendMessageToAll(*mDestroyActor);
		break;
	case '1':
		sendMessageToAll(*mCreateActor);
		break;

	default:
		handled = false;
		break;
	}

	// the default case handles the escape key to quit.
	if (!handled)
		return BaseInputComponent::HandleKeyPressed(keyboard, key);

	return handled;
}

bool SimulationInputComponent::HandleButtonPressed(const dtCore::Mouse* mouse, dtCore::Mouse::MouseButton button)
{														//HandleButtonClicked seems to be broken
	bool handled = true;

	if(button==dtCore::Mouse::LeftButton)									//we have to do it here, and send a message to the GUI Component
		sendMessageToAll(*mMouseLBClick);									//can't do it directly there because of multiple problems
	else
		handled = false;

	if(!handled)
		return BaseInputComponent::HandleButtonPressed(mouse, button);

	return handled;
}

//////////////////////////////////////////////////////////////////////////
void SimulationInputComponent::sendMessageToAll(dtDAL::GameEvent &event)
{
	dtCore::RefPtr<dtGame::GameEventMessage> eventMsg;
	GetGameManager()->GetMessageFactory().CreateMessage(dtGame::MessageType::INFO_GAME_EVENT, eventMsg);

	eventMsg->SetGameEvent(event);
	GetGameManager()->SendMessage(*eventMsg);
}


void SimulationInputComponent::sendMessageToRobot(dtDAL::GameEvent &event)
{
	if(robotControlEnabled)
	{
		dtCore::RefPtr<dtGame::GameEventMessage> eventMsg;
		GetGameManager()->GetMessageFactory().CreateMessage(dtGame::MessageType::INFO_GAME_EVENT, eventMsg);

		eventMsg->SetGameEvent(event);

		//		SimulationRobotSelectionComponent *comp = static_cast<SimulationRobotSelectionComponent*>(GetGameManager()->GetComponentByName("SimulationRobotSelectionComponent"));
		//		int index = comp->getRobFocIndex();
		SimulationArenaComponent *comp = static_cast<SimulationArenaComponent*>(GetGameManager()->GetComponentByName(SimulationArenaComponent::DEFAULT_NAME));
		int index = comp->getRobFocIndex();


		if(index!=-1)
		{
			std::vector< dtGame::GameActorProxy* > allGameActors;
			GetGameManager()->GetAllGameActors(allGameActors);
			//RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[index]->GetGameActor());

			dtCore::RefPtr<RobotActorBase> selectedActor = comp->getRobotsInScene()[index];

			eventMsg->SetAboutActorId(selectedActor->GetUniqueId());
			GetGameManager()->SendMessage(*eventMsg);
		}

	}
}


void SimulationInputComponent::commMessage()
{
	dtCore::UniqueId* busID = new dtCore::UniqueId();

	int r = std::rand() / (RAND_MAX/2);
	int g = std::rand() / (RAND_MAX/2);
	int b = std::rand() / (RAND_MAX/2);

	osg::Vec3d rgb = osg::Vec3d(r,g,b);

	std::stringstream t;
	t << rgb;
	std::string msg = t.str();

	dtCore::RefPtr<CommMessage> commMsg = new CommMessage();
	GetGameManager()->GetMessageFactory().CreateMessage(SimulationMessageType::COMM_BUS, commMsg);

	if (GetGameManager()->GetMessageFactory().IsMessageTypeSupported(commMsg->GetMessageType())) {
		commMsg->setBusID(busID);
		commMsg->setMessage(msg);
		commMsg->setCommType(SimulationMessageType::GLOBAL);
		GetGameManager()->SendMessage(*commMsg);
	}
}

/**
 * Receives two possible messages, COMM_BUS or TICK_LOCAL. This component makes sure that
 * not every image is actually written as a screenshot, but only with a frequency defined
 * by NO_OF_TICKS_UNTIL_SCREENSHOT.
 */
void SimulationInputComponent::ProcessMessage(const dtGame::Message& message)
{
	if (message.GetMessageType() == SimulationMessageType::COMM_BUS)
	{
		//		dtCore::UniqueId* busID = dynamic_cast<CommMessage*>(const_cast<dtGame::Message*>(&message))->getBusID();
		std::string msg = dynamic_cast<CommMessage*>(const_cast<dtGame::Message*>(&message))->getMessage();
		SimulationMessageType::commType type = dynamic_cast<CommMessage*>(const_cast<dtGame::Message*>(&message))->getCommType();

		if (type == SimulationMessageType::ORGANISM) 	rotAroundRobot = true;
	}

	if (message.GetMessageType() == dtGame::MessageType::TICK_LOCAL)
	{
		ticks++;

		if ((takeScreenshot) && (ticks>NO_OF_TICKS_UNTIL_SCREENSHOT))
			dynamic_cast<SimulationCameraController*>(GetGameManager()->GetComponentByName(
					"SimulationCameraController"))->toggleScreenShots(true);

		else {
			ticks = 0;
			dynamic_cast<SimulationCameraController*>(GetGameManager()->GetComponentByName(
					"SimulationCameraController"))->toggleScreenShots(false);
		}

	}

	if (message.GetMessageType() == srCore::SimulationMessageType::COMM_BUS) {
		std::ostringstream printmsg; printmsg.clear(); printmsg.str("");
		LOG_INFO("Received simulation message");

		dtCore::UniqueId* busID = dynamic_cast<CommMessage*>(const_cast<dtGame::Message*>(&message))->getBusID();
		std::string msg = dynamic_cast<CommMessage*>(const_cast<dtGame::Message*>(&message))->getMessage();
		printmsg << "Message \"" << msg << "\" on bus id " << busID->ToString();
		LOG_INFO(printmsg.str());
	}
}

