/**
 * @file parameterReturnValue.cpp
 * @brief 
 *
 * This file is created at Almende B.V. It is open-source software and part of the Common 
 * Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools, ranging from 
 * thread pools and TCP/IP components to control architectures and learning algorithms. 
 * This software is published under the GNU Lesser General Public license (LGPL).
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Nov 22, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	
 */


// General files
#include <sstream>
#include <string>

// Open scene graph
#include <osg/Vec2d>
#include <osg/Vec4>

// Delta3D fiels
#include <dtUtil/log.h>

// Plugin files
#include <srCore/parameterReturnValue.h>

//Adjust this!
using namespace srCore;

/* **************************************************************************************
 * Implementation of ParameterReturnValue
 * **************************************************************************************/

ParameterReturnValue::ParameterReturnValue(const ParameterReturnValue& value):
		type(value.type), pValue(value.pValue) {

}

/**
 * A function that prints the incorrectness of a certain parameter, mainly for
 * debugging purposes.
 */
bool PRVUtils_::isCorrectParameterType(const ParameterReturnValue &parameter,
		const std::string &type) {
	bool result = (parameter.type == type);

	if (!result) {
		bool unknown_type = false;
		std::ostringstream msg; msg.clear(); msg.str("");
		msg << "Parameter type needs to be \"" << type << "\" and is \"";
		msg << parameter.type << "\"";
		LOG_ERROR(msg.str());
		msg.clear(); msg.str("");
		if (parameter.type.compare(typeid(float*).name()) == 0) {
			float *value = reinterpret_cast<float*> (parameter.pValue);
			msg << "Parameter value is: " << *value;
		} else if (parameter.type.compare(typeid(osg::Vec4f*).name()) == 0) {
			osg::Vec4f *value = reinterpret_cast<osg::Vec4f*> (parameter.pValue);
			msg << "Parameter value is: [" << value->_v[0];
			for (int i = 1; i < 4; i++) msg << ',' << value->_v[i];
			msg << ']';
		} else if (parameter.type.compare(typeid(osg::Vec4*).name()) == 0) {
			osg::Vec4 *value = reinterpret_cast<osg::Vec4*> (parameter.pValue);
			msg << "Parameter value is: [" << value->_v[0];
			for (int i = 1; i < 4; i++) msg << ',' << value->_v[i];
			msg << ']';
		} else if (parameter.type.compare(typeid(osg::Vec3f*).name()) == 0) {
			osg::Vec3f *value = reinterpret_cast<osg::Vec3f*> (parameter.pValue);
			msg << "Parameter value is: [" << value->_v[0];
			for (int i = 1; i < 3; i++)  msg << ',' << value->_v[i];
			msg << ']';
		} else if (parameter.type.compare(typeid(osg::Vec2d*).name()) == 0) {
			osg::Vec2d *value = reinterpret_cast<osg::Vec2d*> (parameter.pValue);
			msg << "Parameter value is: [" << value->_v[0] << ',' << value->_v[1] << ']';
		} else {
			unknown_type = true;
		}

		if (!unknown_type) LOG_ERROR(msg.str());
	}
	return result;
}

