/*
 * Component for robot selection
 *
 * @author Adrian Friebel
 */

// Robot3D files
#include <srCore/robotSelectionComponent.h>
#include <srCore/arenaComponent.h>
#include <srCore/robot/robotActorBase.h>
#include <srCore/simulationEvents.h>

using namespace srCore;

////////////////////////////////////////////////////////////////////
SimulationRobotSelectionComponent::SimulationRobotSelectionComponent(const std::string &name, int robFoc)
 : dtGame::GMComponent(name)
{
	mFocusChanged = new dtDAL::GameEvent( ROBOT_SELECTION_CHANGED );
	dtDAL::GameEventManager::GetInstance().AddEvent(*mFocusChanged);

	robotFocus 		= robFoc;
}


void SimulationRobotSelectionComponent::init()
{
	std::vector< dtGame::GameActorProxy* > allGameActors;
	GetGameManager()->GetAllGameActors(allGameActors);

	for(int i=0; i<(int)allGameActors.size(); i++)
	{
		RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[i]->GetGameActor());

		if(robotFocus == actor.getID())
		{
			robotFocusIndex = i;
			actor.setSelectionState(true);
			actor.renderCollisionGeometry(true);
			break;
		}
	}
}


//////////////////////////////////////////////////////////////////////////
void SimulationRobotSelectionComponent::sendMessageToAll(dtDAL::GameEvent &event)
{
   dtCore::RefPtr<dtGame::GameEventMessage> eventMsg;
   GetGameManager()->GetMessageFactory().CreateMessage(dtGame::MessageType::INFO_GAME_EVENT, eventMsg);

   eventMsg->SetGameEvent(event);
   GetGameManager()->SendMessage(*eventMsg);
}


//////////////////////////////////////////////////////////////////////////
void SimulationRobotSelectionComponent::ProcessMessage(const dtGame::Message& message)
{
	if (message.GetMessageType() == dtGame::MessageType::INFO_GAME_EVENT)
	{
		const dtGame::GameEventMessage &eventMsg = static_cast<const dtGame::GameEventMessage&>(message);
//		printf("Robot Selection Component: %s\n", eventMsg.GetGameEvent()->GetName().c_str());

		if(eventMsg.GetGameEvent() != NULL)
		{
		   if(eventMsg.GetGameEvent()->GetName() == CHANGE_ROBOT_SELECTION )

		   {		
			   SimulationArenaComponent *comp = static_cast<SimulationArenaComponent*>(
					   GetGameManager()->GetComponentByName("SimulationArenaComponent"));
			   
			   if(comp->getNumRobots()>0)
			   {
				   std::vector< dtGame::GameActorProxy* > allGameActors;
				   GetGameManager()->GetAllGameActors(allGameActors);

				   RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[robotFocusIndex]->GetGameActor());
				   actor.setSelectionState(false);
				   actor.renderCollisionGeometry(false);

				   std::string number = eventMsg.GetGameEvent()->GetDescription();
				   if(number.empty())
				   {
					   robotFocusIndex = (robotFocusIndex+1) % comp->getNumRobots();
					   RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[robotFocusIndex]->GetGameActor());
					   robotFocus = actor.getID();
					   actor.setSelectionState(true);
					   actor.renderCollisionGeometry(true);
				   }
				   else 
				   {
					   robotFocus = atoi(number.c_str());

					   for(int i=0; i<comp->getNumRobots(); i++)
					   {
						   RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[i]->GetGameActor());


						   if(robotFocus == actor.getID()) {

							   std::ostringstream msg; msg.clear(); msg.str("");
							   msg << "Robot under focus is now: " << actor.getID();
							   LOG_INFO(msg.str());

							   robotFocusIndex = i;
							   actor.setSelectionState(true);
							   actor.renderCollisionGeometry(true);
							   break;
						   }
					   }
				   }
				   sendMessageToAll(*mFocusChanged);
			   }
		   }
		   if(eventMsg.GetGameEvent()->GetName() == DESTROY_ACTOR )
		   {
			   SimulationArenaComponent *comp = static_cast<SimulationArenaComponent*>(GetGameManager()->GetComponentByName("SimulationArenaComponent"));

			   if(comp->getNumRobots()>0)
			   {
				   robotFocusIndex = (robotFocusIndex+1) % comp->getNumRobots();
				   std::vector< dtGame::GameActorProxy* > allGameActors;
				   GetGameManager()->GetAllGameActors(allGameActors);
				   
				   RobotActorBase &actor = static_cast<RobotActorBase&>(allGameActors[robotFocusIndex]->GetGameActor());
				   robotFocus = actor.getID();
				   actor.setSelectionState(true);
				   actor.renderCollisionGeometry(true);			   

			   }
			   else
				  robotFocusIndex = -1;
				  

			   sendMessageToAll(*mFocusChanged);
		   }
		}
	}
}

