#include <srCore/sensor/cameraSensor.h>

#include <dtUtil/log.h>

namespace srCore {

/**
 * Constructor creates camera by default with resolution of 256x256 pixels, RGB
 * type of image, a given flare angle between 0 and PI/4.
 */
CameraSensor::CameraSensor(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3 pos, osg::Vec3 orient,
		float flareAngle, osg::Vec2 resolution, const std::string &name):
	CameraSensorBase(attachedTo, pos, orient, name, "CameraSensor", RGB_CAMERA, true),
	value() {

	std::ostringstream msg;

	minDistance = 1;
	maxDistance = 256;
	this->resolution = resolution;
	this->flareAngle = flareAngle;
	cameraType = RGB_CAMERA;

	if ((flareAngle>0) && (flareAngle != M_PI/4) ) {
		frustumDown = tan(flareAngle/2) * minDistance;
		frustumUp = tan(flareAngle/2) * minDistance;
		frustumRight = tan(flareAngle/2) * minDistance;
		frustumLeft = tan(flareAngle/2) * minDistance;
	} else {
		msg.clear(); msg.str("");
		msg << "The flare angle has an unsupported value: " << flareAngle;
		msg << " (should be between 0 and PI/4)";
		LOG_ERROR(msg.str());
	}

	setup();
	sensorReturnValueType = typeid(osg::Image**).name();

	msg.clear(); msg.str("");
	msg << "Init camera with return value type " << this->sensorReturnValueType;
	LOG_DEBUG(msg.str());
}


/**
 * Destructor
 */
CameraSensor::~CameraSensor() {
	value = NULL;
}

/**
 * This function is called from
 * @param srv
 */
void CameraSensor::evaluate(ParameterReturnValue srv) {

	// sensor value type check
	if (srv.type!=typeid(osg::Image**).name()) {
		std::ostringstream msg; msg.clear(); msg.str("");
		msg << "Parameter type needs to be \"" << typeid(osg::Image**).name() << "\"";
		msg << " and is of type \"" << srv.type.data() << "\". ";
		msg << "Subsequent casting may very well lead to errors.";
		LOG_WARNING(msg.str());
	}

	// cast void* to Image**
	osg::Image** data = reinterpret_cast<osg::Image**> (srv.pValue);

	// create image consisting of the raw data values
	value = new osg::Image(*getRawData());

	// put back these values to the sensor return value
	*data = value;
	srv.pValue = data;
}

}
