/*
 * depthImageCamera.cpp
 *
 *  Created on: Dec 29, 2008
 *      Author: winkler
 */

#include "depthImageCamera.h"

namespace srCore {

DepthImageCamera::DepthImageCamera(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3 pos, osg::Vec3 orient, float flareAngle, osg::Vec2 resolution, const std::string &name):
	CameraSensorBase(attachedTo, pos, orient, name, "DepthImageCamera", DEPTH_CAMERA, true)
{

	// set up camera properties
	this->minDistance = 0.3;
	this->maxDistance = 256;
	this->resolution = resolution;
	this->flareAngle = flareAngle;
	this->cameraType = DEPTH_CAMERA;

	if ((flareAngle>0) && (flareAngle != M_PI/4) ) {

		this->frustumDown = tan(flareAngle/2) * this->minDistance;
		this->frustumUp = tan(flareAngle/2) * this->minDistance;
		this->frustumRight = tan(flareAngle/2) * this->minDistance;
		this->frustumLeft = tan(flareAngle/2) * this->minDistance;

	}

	//this->init(attachedTo, pos, orient);
	this->setup();
	//this->type = "DepthImageCamera";

	this->sensorReturnValueType = typeid(osg::Image**).name();

}


void DepthImageCamera::evaluate(ParameterReturnValue srv)
{

	/*
	osg::Image* image = new osg::Image;

	int x = static_cast<int>(this->camera->GetOSGCamera()->getViewport()->x());
	int y = static_cast<int>(this->camera->GetOSGCamera()->getViewport()->y());
	unsigned int width = static_cast<unsigned int>(this->camera->GetOSGCamera()->getViewport()->width());
	unsigned int height = static_cast<unsigned int>(this->camera->GetOSGCamera()->getViewport()->height());

	printf("%s: Viewport x: %d Viewport y: %d Höhe: %d, Breite: %d \n", this->camera->GetName().c_str(), x, y, height, width);

	image->allocateImage(width, height, 1, GL_RGB, GL_UNSIGNED_BYTE);
	image->readPixels(x, y, width, height, GL_RGB, GL_UNSIGNED_BYTE);
	*/

	//osg::ref_ptr<osg::Image> image = getRawData();
	//srv = MAKE_SENSOR_VALUE(&image);

	// sensor value type check
	if (srv.type!=typeid(osg::Image**).name()) printf("Parameter type needs to be osg::Image**. Casting could lead to errors.  \n "
			"Parameter given: %s \n", srv.type.data());

	// cast void* to Image**
	osg::Image** data = reinterpret_cast<osg::Image**> (srv.pValue);

	// create image consisting of the raw data values
	value = new osg::Image(*getRawData());

	// put back these values to the sensor return value
	*data = value;
	srv.pValue = data;

	//value->unref();
}

}
