#include "laserScanner.h"

namespace srCore {

LaserScanner::LaserScanner(dtCore::RefPtr<BodyBase> attachedTo, osg::Vec3f pos, osg::Vec3f orient, float flareAngle, osg::Vec2 resolution, const std::string &name):
	RaySensorBase(attachedTo, pos, orient, name, "LaserScanner", false)
{

	// set up laser scanner

	this->minDistance = 0.4;
	this->maxDistance = 10;
	this->resolution  = resolution;
	this->flareAngle= flareAngle;
	sensorImage->allocateImage(static_cast<int>(resolution.x()), static_cast<int>(resolution.y()), 1, GL_RGB, GL_UNSIGNED_BYTE);


	if ((flareAngle>0) && (flareAngle != M_PI/4) ) {

		this->frustumDown = tan(flareAngle/2) * this->minDistance;
		this->frustumUp = tan(flareAngle/2) * this->minDistance;
		this->frustumRight = tan(flareAngle/2) * this->minDistance;
		this->frustumLeft = tan(flareAngle/2) * this->minDistance;
#ifdef VERBOSE
		printf("frustumRight: %f", frustumRight);
#endif
	}

	this->setup();
	this->sensorReturnValueType = typeid(osg::Image**).name();

	this->active = false;

	LOG_INFO("Laser Scanner constructed.");

}

void LaserScanner::evaluate(ParameterReturnValue srv)
{

	if (srv.type == typeid(int*).name()) {

		int* data = reinterpret_cast<int*> (srv.pValue);

		// return sensor value
		*data = this->currentSensorValue;
		srv.pValue = data;

		return;
	}

	if (srv.type == typeid(unsigned long*).name()) {

		unsigned long* data = reinterpret_cast<unsigned long*> (srv.pValue);

		// return sensor value
		*data = this->currentSensorValue;
		srv.pValue = data;

		return;
	}

	// type check sensor return value (equals ParameterReturnValue)
	if (srv.type!=typeid(osg::ref_ptr<osg::Image>*).name()) {

		std::ostringstream oss;
		oss << "Parameter type needs to be osg::ref_ptr<osg::Image>* instead of " << srv.type
			<< ". Casting could lead to errors. Parameter given: " << srv.type.data();
		LOG_WARNING(oss.str());
	}

	// cast void* to Image**
	osg::ref_ptr<osg::Image>* data = reinterpret_cast<osg::ref_ptr<osg::Image>*> (srv.pValue);

	// create image consisting of the raw data values
	value = new osg::Image(*getRawData());

	// put back these values to the sensor return value
	*data = value;
	srv.pValue = data;


}


LaserScanner::~LaserScanner()
{
}

}
