/*
 * sensorBase.cpp
 *
 *  Created on: Apr 28, 2009
 *      Author: winkler
 */

// General libraries
#include <iostream>

// Delta3D libraries
#include <dtUtil/log.h>

// Robot3D files
#include <srCore/sensor/sensorBase.h>
#include <srCore/robot/robotActorBase.h>

namespace srCore {



SensorBase::SensorBase(dtCore::RefPtr<BodyBase> attachedBody,
		osg::Vec3f translation, osg::Vec3f orientation,
		std::string name, std::string type, glType cameraType,
		bool active):
			ObjectBase(attachedBody, translation, orientation, name, type, active),
			cameraType		(cameraType),
			isActiveSensor	(true)
{

	std::ostringstream msg;	msg.clear(); msg.str("");
	msg << "Load sensor \"" << name << "\" on robot " << getRobot()->GetName();
	LOG_INFO(msg.str());

	std::ostringstream file;
	file << SimulationUtils::Instance()->getStageProjectPath();
	file << "/StaticMeshes/cone.3ds";
	LoadFile(file.str());
	SetScale(osg::Vec3(0, 0, 0));

	osg::Vec3 scale = this->myBody->getScale();

	SetModelRotation(osg::Vec3(90,90,0));//relOrientation.x(), relOrientation.y()+90, relOrientation.z()));
	SetModelTranslation(osg::Vec3(relTranslation.x() * scale.x(), relTranslation.y() * scale.y(), relTranslation.z() * scale.z()));

	osg::Vec3 data = osg::Vec3(relTranslation.x() * scale.x(), relTranslation.y() * scale.y(), relTranslation.z() * scale.z());
	setParameter("RelPosition", MAKE_PARAMETER_VALUE(&data));

}


SensorBase::~SensorBase() {
	std::ostringstream msg;	msg.clear(); msg.str("");
	msg << "Unload sensor \"" << name << "\" on robot " << getRobot()->GetName();
	LOG_INFO(msg.str());
}

void SensorBase::makeVisible(bool visible) {

	if (visible) {
		osg::Vec3 scale = this->myBody->getScale();
		SetScale(osg::Vec3(scale.x()/10, scale.y()/10, scale.z()/10));
		SetModelRotation(osg::Vec3(90,90,0));//relOrientation.x(), relOrientation.y()+90, relOrientation.z()));
		SetModelTranslation(osg::Vec3(relTranslation.x() * scale.x(), relTranslation.y() * scale.y(), relTranslation.z() * scale.z()));
	}

	else SetScale(osg::Vec3(0, 0, 0));

}


dtGame::GameManager* SensorBase::getGameManager() const
{
	return getRobot()->GetGameActorProxy().GetGameManager();
}

std::ostream& operator<< (std::ostream& theStream, const SensorBase& sensor)
{
  sensor.getSensorValue(theStream);
  return theStream;
}

}
