/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */

#include <srFactory/actuatorFactory.h>

namespace srCore {


dtCore::RefPtr<ActuatorBase> ActuatorFactory::createActuator(actuatortype type,
		dtCore::RefPtr<BodyBase> body1, dtCore::RefPtr<BodyBase> body2,
		float anchorX, float anchorY, float anchorZ,
		float axisX, float axisY, float axisZ,
		const std::string &name)
{

	switch (type)
	{

	case HINGE_ACTUATOR:

		return new HingeUniKarl(
				body1, body2,
				anchorX, anchorY, anchorZ,
				axisX, axisY, axisZ, name);

	case WHEEL_ACTUATOR:
		return new WheelUniKarl(
				body1, body2,
				anchorX, anchorY, anchorZ,
				axisX, axisY, axisZ, name);

	default:
		return NULL;

	}

}

ActuatorFactory::~ActuatorFactory() {

}

}
