/**
 * @file YarpPlugin.cpp
 *
 * This file YarpPlugin.cpp is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license,
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author 	Anne C. van Rossum
 * @date	Aug 4, 2010
 * @project	Replicator FP7
 * @company	Almende B.V.
 * @case	YARP
 */


// General files
#include <string>

// Delta3D files
#include <dtUtil/log.h>

// Plugin files
#include <srInterface/YarpPlugin.h>
#include <srCore/export.h>
#include <srInterface/InterfaceUtil.h>

/// This entry point is necessary to be loaded by the Robot3D simulator
extern "C" ROBOT_EXPORT srInterface::YarpPlugin* CreatePluginEntryPoint() {
	return YarpPluginSingleton::Instance();
}

/// This entry point is necessary to be deallocated by the Robot3D simulator
extern "C" ROBOT_EXPORT void DestroyPluginEntryPoint() {
	// destroying a singleton?
}

namespace srInterface {

/* **************************************************************************************
 * Implementation of YarpPlugin
 * **************************************************************************************/

YarpPlugin::YarpPlugin(): yarped_sim(NULL) {

}

YarpPlugin::~YarpPlugin() {

}

/**
 *
 */
void YarpPlugin::StartPlugin(dtGame::GameApplication &application) {
	dtUtil::Log::LogMessageType level = dtUtil::Log::GetInstance().GetLogLevel();
	dtUtil::Log::GetInstance().SetLogLevel(dtUtil::Log::LOG_DEBUG);

	LOG_DEBUG("Loading plugin");

	yarped_sim = new YarpedSim();
	application.GetGameManager()->AddComponent(*yarped_sim, dtGame::GameManager::ComponentPriority::NORMAL);

	srCore::Robot3DComponent* comp = InterfaceUtil::GetRobot3DComponent();

	std::vector<std::string> plugin_controllers;
	std::vector<std::string>::iterator it;
	plugin_controllers.clear();
	plugin_controllers.push_back("YarpedRobot");
	plugin_controllers.push_back("NavigationController");
	plugin_controllers.push_back("ZigbeeController");

	for (unsigned int i=0; i < comp->GetConfigObject().getControllers().size(); i++) {
		ControllerInitData cid = comp->GetConfigObject().getControllers()[i];
		for (it = plugin_controllers.begin(); it != plugin_controllers.end(); it++) {
			if ((*it).compare(cid.type) == 0) {
				yarped_sim->AddController(cid);
				break;
			}
		}
	}

	LOG_DEBUG("Plugin loaded");

	dtUtil::Log::GetInstance().SetLogLevel(level);
}

}
