/*
 * Robot3D Physical realistic 3D simulator for robots
 *
 * @license GNU Lesser General Public License
 *
 * @author Matthew W. Campbell
 * @author Lutz Winkler
 * @author Anne C. van Rossum
 */

// Robot3D files
#include <srRobot/robotRegistry.h>

// Delta3D files
#include <dtUtil/log.h>

// The default robots (no user-defined ones)
#include <srRobot/robotKIT.h>
#include <srRobot/robotKIT2.h>

namespace srRobot {

/**
 * The bare C hooks needed for Delta3D to get grips on this registry
 */
extern "C" ROBOT_EXPORT dtDAL::ActorPluginRegistry* CreatePluginRegistry() {
   return new RobotRegistry;
}

/**
 * A C handle to remove it.
 */
extern "C" ROBOT_EXPORT void DestroyPluginRegistry(dtDAL::ActorPluginRegistry *registry) {
   if(registry) delete registry;
}

/**
 * Robot from Karlsruhe Institute of Technology.
 * @todo It would be nicer to use RobotKIT::DEFAULT_NAME instead of a hardcoded "RobotKIT" string, but
 * this leads to a segmentation fault.
 */
dtCore::RefPtr<dtDAL::ActorType> RobotRegistry::RobotKITType (
		new dtDAL::ActorType("RobotKIT", "RobotKIT", "A KIT robot"));

dtCore::RefPtr<dtDAL::ActorType> RobotRegistry::RobotKIT2Type (
		new dtDAL::ActorType("RobotKIT2", "RobotKIT2", "A KIT robot, with a different implementation of the screw drive"));


/**
 * The registry as how it needs to be implemented in Delta3D is old-fashioned and hard-coded. For that
 * reason there should be an additional method to add custom robots to the registry. For now I see only as
 * a solution to have user-defined registries.
 *
 * @url http://www.gamedev.net/reference/articles/article2097.asp
 */
RobotRegistry::RobotRegistry() : dtDAL::ActorPluginRegistry("SimulationRobotActors"),
		initialised(true) {
	SetDescription("The registry for robots");
	LOG_INFO("The registry for robots is online");
}

/**
 * Register all known actors.
 */
void RobotRegistry::RegisterActorTypes() {
	LOG_INFO("Register the actors in the Robot3D simulator");
	mActorFactory->RegisterType<RobotKITProxy>(RobotKITType.get());
	mActorFactory->RegisterType<RobotKIT2Proxy>(RobotKIT2Type.get());
}

} // end of namespace srCore
