/*
 * sampleSensor.cpp
 *
 *  Created on: Nov 28, 2008
 *      Author: winkler
 */

#include <srSensor/sampleSensor.h>

namespace srCore {

SampleSensor::SampleSensor(dtCore::RefPtr<BodyBase> attachedBody,
		osg::Vec3f translation, osg::Vec3f orientation,
		std::string name,
		bool active) :

		SensorBase(attachedBody,  translation, orientation, name, "SampleSensor", NO_CAMERA, active),
		sensorValue (0)

{
	this->sensorReturnValueType = typeid(float*).name();

}

SampleSensor::~SampleSensor() {
	// TODO Auto-generated destructor stub
}

void SampleSensor::getParameter(const std::string &name, ParameterReturnValue  parameter) {

	ObjectBase::getParameter(name, parameter);

}

void SampleSensor::setParameter(const std::string &name, ParameterReturnValue  parameter) {

	ObjectBase::setParameter(name, parameter);
}

void SampleSensor::evaluate(ParameterReturnValue srv) {
	//check right type
	if (srv.type!=this->sensorReturnValueType) {
		std::ostringstream msg; msg.clear(); msg.str("");
		msg << "Parameter type needs to be " << sensorReturnValueType;
		msg << " while given type is " << srv.type;
		LOG_ERROR(msg.str());
	}

	float* data = reinterpret_cast<float*> (srv.pValue);
	*data = this->sensorValue;
}


void SampleSensor::requestData() {

	osg::Vec3f pos, rot;

	pos = osg::Vec3f(5,5,5); rot = osg::Vec3f(5,5,5);
	getParameter("AbsPosition", MAKE_PARAMETER_VALUE(&pos));
	//getParameter("AbsOrientation", MAKE_PARAMETER_VALUE(&rot));

	sensorValue = myFancySimulatedEnvironmentFunction(pos, rot);

}

void SampleSensor::getNewSensorValue() {

	requestData();

}

void SampleSensor::getSensorValue(std::ostream& theStream) const {

	theStream << sensorValue;

}

float SampleSensor::myFancySimulatedEnvironmentFunction(osg::Vec3f pos, osg::Vec3f rot) {

	return (pos - osg::Vec3f(20,15,0)).length();

}

void SampleSensor::update(float deltaSimTme) {

	if (active && isActiveSensor) requestData();

}

}
