/**
 * @file LoadLibrary.cpp
 * @brief Tester class for loading a library. Might be outphased.
 *
 * This file LoadLibrary.cpp is created at Almende B.V. It is open-source software and part
 * of the Common Hybrid Agent Platform (CHAP). A toolbox with a lot of open-source tools.
 * Ranging from thread pools, and TCP/IP components to control architectures and learning
 * algorithms. This software is published under the GNU Lesser General Public license.
 *
 * It is not possible to add usage restrictions to an open-source license. Nevertheless,
 * we personally strongly object against this software used by the military, in the
 * bio-industry, for animal experimentation, or anything that violates the Universal
 * Declaration of Human Rights.
 *
 * Copyright © 2010 Anne van Rossum <anne@almende.com>
 *
 * @author  Anne C. van Rossum
 * @date    Dec 2, 2009
 * @project Replicator FP7
 * @company Almende B.V.
 */

#include <cstdio>
#include <dlfcn.h>
#include <dtGame/gameapplication.h>
#include <dtUtil/log.h>
#include <sys/stat.h>

/**
* <Plugin name="libplugins_delta3d.so" path="/home/anne/myworkspace/replicator/plugins_delta3d/Debug" />
*/
int main() {
	dtUtil::Log::GetInstance().SetLogLevel(dtUtil::Log::LOG_INFO);
	std::ostringstream msg;
	msg.clear(); msg.str("");
	char filename[64] = "libplugins_delta3d.so";
	char filepath[128] = "/home/anne/myworkspace/replicator/plugins_delta3d/Debug";
	msg << "Plugin \"" << filename << "\"";
	msg << " at path " << filepath << "";
	LOG_INFO(msg.str());
	char fullfilename[256];
	sprintf(fullfilename, "%s/%s", filepath, filename);
	msg.clear(); msg.str("");
	msg << "Full file name: " << fullfilename;
	LOG_INFO(msg.str());
	struct stat stFileInfo;
	if (stat(fullfilename,&stFileInfo)) {
		msg.clear(); msg.str("");
		msg << "No such file!";
		LOG_ERROR(msg.str());
		return -1;
	}

	void* handle = dlopen(fullfilename, RTLD_LAZY);
	if (!handle) {
		msg.clear(); msg.str("");
		msg << "No such file? (" << dlerror() << ")";
		LOG_ERROR(msg.str());
		return -1;
	}

	void (*entrypoint)(dtGame::GameApplication&);
	entrypoint = (void (*)(dtGame::GameApplication&))dlsym(
			handle, //"_ZN16PluginEntryPoint11StartPluginERN6dtGame15GameApplicationE");
			"PluginEntryPoint::StartPlugin(dtGame::GameApplication&)");
	char *error;
	if ((error = dlerror()) != NULL) {
		msg.clear(); msg.str("");
		msg << "No StartPlugin method found! (" << error << ")";
		LOG_ERROR(msg.str());
		return -1;
	}

	//(*entrypoint)(app);
	dlclose(handle);

	return 0;
}
